
import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterVmGroup(vim, *args, **kwargs):
    '''The ClusterVmGroup data object identifies virtual machines for VM-Host rules.
    VM-Host rules determine placement of virtual machines on hosts in a cluster.
    The logic specified in a ClusterVmHostRuleInfo object determines where virtual
    machines can be powered-on.If a virtual machine is removed from the cluster, it
    loses its DRS group affiliation. The Server does not restore any group
    affiliations if the virtual machine is returned to the cluster.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterVmGroup')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'name' ]
    optional = [ 'vm', 'userCreated', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpConnection(Base):
    '''
    List active tcpip connections
    '''
    moid = 'ha-cli-handler-network-ip-connection'
    def list(self, type=None):
        '''
        List active TCP/IP connections
        :param type: string, Connection type :  [ip, tcp, udp, all]
        :returns: vim.EsxCLI.network.ip.connection.list.IpConnection[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.connection.List',
                            type=type,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterFirmware(Base):
    '''
    Operations that can be performed on iSCSI adapter firmware
    '''
    moid = 'ha-cli-handler-iscsi-adapter-firmware'
    def set(self, adapter, file):
        '''
        Upload the iSCSI firmware for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param file: string, Path to the firmware file to download.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.firmware.Set',
                            adapter=adapter,
                            file=file,
                            )
    def get(self, adapter, file):
        '''
        Validate the iSCSI firmware for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param file: string, Path to the firmware file to retrieve information from.
        :returns: vim.EsxCLI.iscsi.adapter.firmware.get.AdapterFirmware[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.firmware.Get',
                            adapter=adapter,
                            file=file,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SelectionSpec(vim, *args, **kwargs):
    '''The SelectionSpec is the base type for data object types that specify what
    additional objects to filter. The base type contains only an optional "name"
    field, which allows a selection to be named for future reference. More
    information is available in the subtype.Named selections support recursive
    specifications on an object hierarchy. When used by a derived object, the
    "name" field allows other SelectionSpec objects to refer to the object by name.
    When used as the base type only, the "name" field indicates recursion to the
    derived object by name.Names are meaningful only within the same FilterSpec.'''

    obj = vim.client.factory.create('{urn:vim25}SelectionSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'name', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ScsiLunState = Enum(
    'degraded',
    'error',
    'lostCommunication',
    'off',
    'ok',
    'quiesced',
    'unknownState',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class VirtualMachineCompatibilityChecker(BaseEntity):
    '''A singleton managed object that can answer questions about compatibility of a
    virtual machine with a host.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.VirtualMachineCompatibilityChecker):
        super(VirtualMachineCompatibilityChecker, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CheckCompatibility_Task(self, vm, host=None, pool=None, testType=None):
        '''Tests whether or not a virtual machine could be placed on the given host in the
        given resource pool.
        
        :param vm: The virtual machine we'd like to place.
        
        :param host: The host we would like the virtual machine to execute on. The host parameter may be left unset if the compute resource associated with the pool represents a stand-alone host or a DRS-enabled cluster. In the former case the stand-alone host is used. In the latter case, each connected host in the cluster that is not in maintenance mode is tested. If the virtual machine is a template then either this parameter or the pool parameter must be set.
        
        :param pool: The resource pool we would like the virtual machine to reside in. If the pool parameter is left unset, then the virtual machine's current pool is assumed. If the virtual machine is a template then either this parameter or the host parameter must be set.
        
        :param testType: The set of tests to run. If this argument is not set, all tests will be run. See CheckTestType for possible values.
        
        '''
        return self.delegate("CheckCompatibility_Task")(vm, host, pool, testType)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostCpuPackageVendor = Enum(
    'amd',
    'intel',
    'unknown',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageNmpDevice(Base):
    '''
    Operations pertaining to the devices currently claimed by the VMware Native Multipath Plugin.
    '''
    moid = 'ha-cli-handler-storage-nmp-device'
    def set(self, device, default=None, psp=None):
        '''
        Allow setting of the Path Selection Policy (PSP) for the given device to one of the loaded policies on the system.
        :param default: boolean, The Path selection policy is set back to the default for the assigned SATP for this device.
        :param device: string, The device you wish to set the Path Selection Policy for.
        :param psp: string, The Path selection policy you wish to assign to the given device.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.device.Set',
                            default=default,
                            device=device,
                            psp=psp,
                            )
    def list(self, device=None):
        '''
        List the devices currently controlled by the VMware NMP Multipath Plugin and show the SATP and PSP information associated with that device.
        :param device: string, Filter the output of this command to only show a single device.
        :returns: vim.EsxCLI.storage.nmp.device.list.NmpDevice[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.device.List',
                            device=device,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

AutoStartAction = Enum(
    'guestShutdown',
    'none',
    'powerOff',
    'powerOn',
    'suspend',
    'systemDefault',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCorePath(Base):
    '''
    Operations pertaining to the pluggable storage architectures' SCSI device paths on the system.
    '''
    moid = 'ha-cli-handler-storage-core-path'
    def set(self, path, state):
        '''
        Provide control to allow a user to modify a single path's state. This efffectively allows a user to enable or disable SCSI paths. The user is not able to change the full range of path states, but can toggle between 'active' and 'off'. Please NOTE changing the Path state on any path that is the only path to a given device is likely to fail. The VMkernel will not change the path's state if changing the state would cause an 'All paths down' state or the device is currently in use.
        :param path: string, Select the path to set path state on. This can be a Runtime Name or Path UID
        :param state: string, Set the SCSI path state for a the specific path given. Valid values are : 
    active: Set the path's state to active. This may be immediately changed by the system to another state if the active state is not appropriate.
    off: Administratively disable this path.

        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.path.Set',
                            path=path,
                            state=state,
                            )
    def list(self, device=None, path=None):
        '''
        List all the SCSI paths on the system.
        :param device: string, Limit the output to paths to a specific device.   This name can be any of the UIDs for a specific device.
        :param path: string, Limit the output to a specific path. This  name can be either the UID or the runtime name of the path.
        :returns: vim.EsxCLI.storage.core.path.list.ScsiPath[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.path.List',
                            device=device,
                            path=path,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ScsiLunType = Enum(
    'cdrom',
    'communications',
    'disk',
    'enclosure',
    'mediaChanger',
    'opticalDevice',
    'printer',
    'processor',
    'scanner',
    'storageArrayController',
    'tape',
    'unknown',
    'worm',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.managed_entity import ManagedEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class Folder(ManagedEntity):
    '''The Folder managed object is a container for storing and organizing inventory
    objects. Folders can contain folders and other objects. The childType property
    identifies a folder's type and determines the types of folders and objects the
    folder can contain.* A folder can contain a child folder of the same type as
    the parent folder. * All Datacenter objects contain dedicated folders for:See
    ServiceInstance for a representation of the organization of the inventory.The
    Folder managed object also acts as a factory object, meaning it creates new
    entities in a folder. The object provides methods to create child folders and
    objects, methods to add existing objects to folders, and methods to remove
    objects from folders and to delete folders.Folder inherits the Destroy_Task
    method. Destroy_Task is a recursive operation that removes all child objects
    and folders. When you call Destroy_Task to destroy a folder, the system uses
    the specified folder as a root and traverses its descendant hierarchy, calling
    Destroy_Task on each object. Destroy_Task is a single operation that treats
    each recursive call as a single transaction, committing each call to remove an
    object individually. If Destroy_Task fails on an object, the method terminates
    at that point with an exception, leaving some or all of the objects still in
    the inventory.Notes on the folder destroy method:'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.Folder):
        super(Folder, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def childEntity(self):
        '''An array of managed object references. Each entry is a reference to a child
        entity.'''
        return self.update('childEntity')
    @property
    def childType(self):
        '''Specifies the object types a folder may contain. When you create a folder, it
        inherits its childType from the parent folder in which it is created. childType
        is an array of strings. Each array entry identifies a set of object types -
        Folder and one or more managed object types. The following list shows childType
        values for the different folders: * { "vim.Folder", "vim.Datacenter" } -
        Identifies the root folder and its descendant folders. Data center folders can
        contain child data center folders and Datacenter managed objects. Datacenter
        objects contain virtual machine, compute resource, network entity, and
        datastore folders. * { "vim.Folder", "vim.Virtualmachine", "vim.VirtualApp" } -
        Identifies a virtual machine folder. A virtual machine folder may contain child
        virtual machine folders. It also can contain VirtualMachine managed objects,
        templates, and VirtualApp managed objects. * { "vim.Folder",
        "vim.ComputeResource" } - Identifies a compute resource folder, which contains
        child compute resource folders and ComputeResource hierarchies. * {
        "vim.Folder", "vim.Network" } - Identifies a network entity folder. Network
        entity folders can contain Network, and DistributedVirtualSwitch managed
        objects. * { "vim.Folder", "vim.Datastore" } - Identifies a datastore folder.
        Datastore folders can contain child datastore folders and Datastore managed
        objects.'''
        return self.update('childType')

    
    
    def AddStandaloneHost_Task(self, spec, addConnected, compResSpec=None, license=None):
        '''Creates a new single-host compute resource. The name provided can be an IP
        address, such as 192.168.0.120, or a string, such as esx120. If a name is
        specified, a DNS lookup is used to resolve it to a fully-qualified name, such
        as esx120.vmware.com. If the DNS lookup fails, the string is stored as
        specified.Creates a new single-host compute resource. The name provided can be
        an IP address, such as 192.168.0.120, or a string, such as esx120. If a name is
        specified, a DNS lookup is used to resolve it to a fully-qualified name, such
        as esx120.vmware.com. If the DNS lookup fails, the string is stored as
        specified.
        
        :param spec: The host name, port, and passwords for the host to be added.
        
        :param compResSpec: Optionally specify the configuration for the compute resource that will be created to contain the host.VI API 2.5
        
        :param addConnected: Flag to specify whether or not the host should be connected as soon as it is added. The host will not be added if a connection attempt is made and fails.
        
        :param license: Provide a licenseKey or licenseKeyType. See LicenseManagervSphere API 4.0
        
        '''
        return self.delegate("AddStandaloneHost_Task")(spec, compResSpec, addConnected, license)
    
    def CreateCluster(self, name, spec):
        '''<b>Deprecated.</b> <i>As of VI API 2.5, use CreateClusterEx.</i> Creates a new
        cluster compute resource in this folder.
        
        :param name: Name for the new cluster.
        
        :param spec: Specification for the cluster.
        
        '''
        return self.delegate("CreateCluster")(name, spec)
    
    def CreateClusterEx(self, name, spec):
        '''Creates a new cluster compute resource in this folder.Creates a new cluster
        compute resource in this folder.
        
        :param name: Name for the new cluster.
        
        :param spec: Specification for the cluster.
        
        '''
        return self.delegate("CreateClusterEx")(name, spec)
    
    def CreateDatacenter(self, name):
        '''Creates a new datacenter with the given name.Creates a new datacenter with the
        given name.
        
        :param name: Name for the new datacenter. An entity name must be a non-empty string of less than 80 characters. The slash (/), backslash (\) and percent (%) will be escaped using the URL syntax. For example, %2F.
        
        '''
        return self.delegate("CreateDatacenter")(name)
    
    def CreateDVS_Task(self, spec):
        '''Create a Distributed Virtual Switch in the folder according to the specified
        (@link vim.DistributedVirtualSwitch.CreateSpec). Folder must be a Network
        entity folder.
        
        :param spec: The (@link vim.DistributedVirtualSwitch.CreateSpec) to create the Distributed Virtual Switch.
        
        '''
        return self.delegate("CreateDVS_Task")(spec)
    
    def CreateFolder(self, name):
        '''Creates a new sub-folder with the specified name. The childType property of the
        new folder is the same as the childType property of the current folder.Creates
        a new sub-folder with the specified name. The childType property of the new
        folder is the same as the childType property of the current folder.
        
        :param name: The name to be given the new folder. An entity name must be a non-empty string of less than 80 characters. The slash (/), backslash (\) and percent (%) will be escaped using the URL syntax. For example, %2F.
        
        '''
        return self.delegate("CreateFolder")(name)
    
    def CreateStoragePod(self, name):
        '''Creates a new storage pod in this folder.Creates a new storage pod in this
        folder.
        
        :param name: Name for the new storage pod.
        
        '''
        return self.delegate("CreateStoragePod")(name)
    
    def CreateVM_Task(self, config, pool, host=None):
        '''Creates a new virtual machine in the current folder and attaches it to the
        specified resource pool. This operation creates a virtual machine, instead of
        cloning a virtual machine from an existing one.Creates a new virtual machine in
        the current folder and attaches it to the specified resource pool. This
        operation creates a virtual machine, instead of cloning a virtual machine from
        an existing one.Creates a new virtual machine in the current folder and
        attaches it to the specified resource pool. This operation creates a virtual
        machine, instead of cloning a virtual machine from an existing one.
        
        :param config: The configuration of the virtual machine hardware.
        
        :param pool: The resource pool to which the virtual machine will be attached.
        
        :param host: The target host on which the virtual machine will run. This must specify a host that is a member of the ComputeResource indirectly specified by the pool. For a stand-alone host or a cluster with DRS, host can be omitted, and the system selects a default.
        
        '''
        return self.delegate("CreateVM_Task")(config, pool, host)
    
    def MoveIntoFolder_Task(self, list):
        '''Moves a set of managed entities into this folder.Moves a set of managed
        entities into this folder.Moves a set of managed entities into this
        folder.Moves a set of managed entities into this folder.Moves a set of managed
        entities into this folder.Moves a set of managed entities into this
        folder.Moves a set of managed entities into this folder.Moves a set of managed
        entities into this folder.Moves a set of managed entities into this
        folder.Moves a set of managed entities into this folder.
        
        :param list: The list of objects to be moved into the folder.
        
        '''
        return self.delegate("MoveIntoFolder_Task")(list)
    
    def RegisterVM_Task(self, path, asTemplate, name=None, pool=None, host=None):
        '''Adds an existing virtual machine to the folder.Adds an existing virtual machine
        to the folder.Adds an existing virtual machine to the folder.
        
        :param path: A datastore path to the virtual machine.
        
        :param name: The name to be assigned to the virtual machine. If this parameter is not set, the displayName configuration parameter of the virtual machine is used. An entity name must be a non-empty string of less than 80 characters. The slash (/), backslash (\) and percent (%) will be escaped using the URL syntax. For example, %2F.
        
        :param asTemplate: Flag to specify whether or not the virtual machine should be marked as a template.
        
        :param pool: The resource pool to which the virtual machine should be attached. If imported as a template, this parameter is not set.
        
        :param host: The target host on which the virtual machine will run. This parameter must specify a host that is a member of the ComputeResource indirectly specified by the pool. For a stand-alone host or a cluster with DRS, the parameter can be omitted, and the system selects a default.
        
        '''
        return self.delegate("RegisterVM_Task")(path, name, asTemplate, pool, host)
    
    def UnregisterAndDestroy_Task(self):
        '''Recursively unregisters all virtual machines and vApps, and destroys all child
        virtual machine folders. This is similar to the Destroy_Task method, but this
        method calls UnregisterAndDestroy_Task on each virtual machine object instead
        of calling Destroy_Task. This operation applies only to VirtualMachine
        folders.Recursively unregisters all virtual machines and vApps, and destroys
        all child virtual machine folders. This is similar to the Destroy_Task method,
        but this method calls UnregisterAndDestroy_Task on each virtual machine object
        instead of calling Destroy_Task. This operation applies only to VirtualMachine
        folders.Recursively unregisters all virtual machines and vApps, and destroys
        all child virtual machine folders. This is similar to the Destroy_Task method,
        but this method calls UnregisterAndDestroy_Task on each virtual machine object
        instead of calling Destroy_Task. This operation applies only to VirtualMachine
        folders.
        
        '''
        return self.delegate("UnregisterAndDestroy_Task")()



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ArrayUpdateOperation = Enum(
    'add',
    'edit',
    'remove',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostFeatureVersionKey = Enum(
    'faultTolerance',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualDiskFlatVer2BackingInfo(vim, *args, **kwargs):
    '''This data object type contains information about backing a virtual disk using a
    virtual disk file on the host, in the flat file format used by VMware Server,
    ESX Server 2.x, and ESX Server 3.x. Flat disks are allocated when created,
    unlike sparse disks, which grow as needed.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualDiskFlatVer2BackingInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'diskMode', 'fileName' ]
    optional = [ 'changeId', 'contentId', 'deltaDiskFormat', 'digestEnabled', 'eagerlyScrub',
        'parent', 'split', 'thinProvisioned', 'uuid', 'writeThrough', 'datastore',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class SearchIndex(BaseEntity):
    '''The SearchIndex service allows a client to efficiently query the inventory for
    a specific managed entity by attributes such as UUID, IP address, DNS name, or
    datastore path. Such searches typically return a VirtualMachine or a
    HostSystem. While searching, only objects for which the user has sufficient
    privileges are considered. The findByInventoryPath and findChild operations
    only search on entities for which the user has view privileges; all other
    SearchIndex find operations only search virtual machines and hosts for which
    the user has read privileges. If the user does not have sufficient privileges
    for an object that matches the search criteria, that object is not returned.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.SearchIndex):
        super(SearchIndex, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def FindAllByDnsName(self, dnsName, vmSearch, datacenter=None):
        '''Finds all virtual machines or hosts by DNS name. The DNS name for a virtual
        machine is the one returned from VMware tools, hostName.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param dnsName: The fully qualified domain name to find.
        
        :param vmSearch: If true, search for virtual machines, otherwise search for hosts.
        
        '''
        return self.delegate("FindAllByDnsName")(datacenter, dnsName, vmSearch)
    
    def FindAllByIp(self, ip, vmSearch, datacenter=None):
        '''Finds all virtual machines or hosts by IP address, where the IP address is in
        dot-decimal notation. For example, 10.17.12.12. The IP address for a virtual
        machine is the one returned from VMware tools, ipAddress.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param ip: The dot-decimal notation formatted IP address to find.
        
        :param vmSearch: If true, search for virtual machines, otherwise search for hosts.
        
        '''
        return self.delegate("FindAllByIp")(datacenter, ip, vmSearch)
    
    def FindAllByUuid(self, uuid, vmSearch, datacenter=None, instanceUuid=None):
        '''Finds all virtual machines or hosts by UUID.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param uuid: The UUID to find. If vmSearch is true, the UUID can be either BIOS or instance UUID.
        
        :param vmSearch: If true, search for virtual machines, otherwise search for hosts.
        
        :param instanceUuid: Should only be set when vmSearch is true. If specified, search for virtual machines whose instance UUID matches the given uuid. Otherwise, search for virtual machines whose BIOS UUID matches the given uuid.
        
        '''
        return self.delegate("FindAllByUuid")(datacenter, uuid, vmSearch, instanceUuid)
    
    def FindByDatastorePath(self, datacenter, path):
        '''Finds a virtual machine by its location on a datastore.
        
        :param datacenter: Specifies the datacenter to which the datastore path belongs.
        
        :param path: A datastore path to the .vmx file for the virtual machine.
        
        '''
        return self.delegate("FindByDatastorePath")(datacenter, path)
    
    def FindByDnsName(self, dnsName, vmSearch, datacenter=None):
        '''Finds a virtual machine or host by DNS name. The DNS name for a virtual machine
        is the one returned from VMware tools, hostName.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param dnsName: The fully qualified domain name to find.
        
        :param vmSearch: if true, search for virtual machines, otherwise search for hosts.
        
        '''
        return self.delegate("FindByDnsName")(datacenter, dnsName, vmSearch)
    
    def FindByInventoryPath(self, inventoryPath):
        '''Finds a managed entity based on its location in the inventory. The path is
        separated by slashes ('/'). For example, a path should be of the form "My
        Folder/My Datacenter/vm/Discovered VM/VM1". A leading slash or trailing slash
        is ignored. Thus, the following paths all represents the same object: "a/b",
        "/a/b", "a/b/", and '/a/b/'. Slashes in names must be represented using %2f,
        following the standard URL syntax. Any object in the inventory can be retrieved
        using this method, including resource pools and hosts.
        
        :param inventoryPath: The path to the entity.
        
        '''
        return self.delegate("FindByInventoryPath")(inventoryPath)
    
    def FindByIp(self, ip, vmSearch, datacenter=None):
        '''Finds a virtual machine or host by IP address, where the IP address is in dot-
        decimal notation. For example, 10.17.12.12. The IP address for a virtual
        machine is the one returned from VMware tools, ipAddress.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param ip: The dot-decimal notation formatted IP address to find.
        
        :param vmSearch: if true, search for virtual machines, otherwise search for hosts.
        
        '''
        return self.delegate("FindByIp")(datacenter, ip, vmSearch)
    
    def FindByUuid(self, uuid, vmSearch, datacenter=None, instanceUuid=None):
        '''Finds a virtual machine or host by BIOS or instance UUID.
        
        :param datacenter: If specified, restricts the query to entities in a particular datacenter. If not specified, the entire inventory is searched.
        
        :param uuid: The UUID to find. If vmSearch is true, the uuid can be either BIOS or instance UUID.
        
        :param vmSearch: If true, search for virtual machines, otherwise search for hosts.
        
        :param instanceUuid: Should only be set when vmSearch is true. If specified, search for virtual machines whose instance UUID matches the given uuid. Otherwise, search for virtual machines whose BIOS UUID matches the given uuid.vSphere API 4.0
        
        '''
        return self.delegate("FindByUuid")(datacenter, uuid, vmSearch, instanceUuid)
    
    def FindChild(self, entity, name):
        '''Finds a particular child based on a managed entity name. This only searches the
        immediate children of a managed entity. For a Datacenter, the host and vm
        folders are considered children. For a ComputeResource, the hosts and root
        ResourcePool are considered children.
        
        :param entity: A reference to a managed entity.
        
        :param name: The name of the child object.
        
        '''
        return self.delegate("FindChild")(entity, name)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkVswitchStandardPolicyShaping(Base):
    '''
    Commands to manipulate shaping policy setting for the given virtual switch.
    '''
    moid = 'ha-cli-handler-network-vswitch-standard-policy-shaping'
    def set(self, vswitchname, avgbandwidth=None, burstsize=None, enabled=None, peakbandwidth=None):
        '''
        Set the shaping policy settings for the given virtual switch
        :param avgbandwidth: long, The averge bandwidth allowed for this shaping policy. This value is in Kbps (1 Kbps = 1000 bits/s)
        :param burstsize: long, The largest burst size allowed for this shaping policy. This value is in Kib (1 Kib = 1024 bits)
        :param enabled: boolean, Indicate whether to enable traffic shaping on this policy. If this is true then the --avg-bandwidth, --peak-bandwidth and --burst-size options are required.
        :param peakbandwidth: long, The peak bandwidth allowed for this shaping policy. This value is in Kbps (1 Kbps = 1000 bits/s)
        :param vswitchname: string, The name of the virtual switch to use when setting the switch shaping policy.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.policy.shaping.Set',
                            avgbandwidth=avgbandwidth,
                            burstsize=burstsize,
                            enabled=enabled,
                            peakbandwidth=peakbandwidth,
                            vswitchname=vswitchname,
                            )
    def get(self, vswitchname):
        '''
        Get the shaping policy settings for the given virtual switch
        :param vswitchname: string, The name of the virtual switch to use when fetching the switch shaping policy.
        :returns: vim.EsxCLI.network.vswitch.standard.policy.shaping.get.VirtualSwitchTrafficShapingPolicy
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.policy.shaping.Get',
                            vswitchname=vswitchname,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapter(Base):
    '''
    Operations that can be performed on iSCSI adapters
    '''
    moid = 'ha-cli-handler-iscsi-adapter'
    def set(self, adapter, alias=None, name=None):
        '''
        Set the iSCSI name and alias for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :param alias: string, The iSCSI initiator alias.
        :param name: string, The iSCSI initiator name.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.Set',
                            adapter=adapter,
                            alias=alias,
                            name=name,
                            )
    def list(self, adapter=None):
        '''
        List all the iSCSI Host Bus Adapters on the system.
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.adapter.list.iScsiAdapter[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.List',
                            adapter=adapter,
                            )
    def get(self, adapter):
        '''
        List the iSCSI information for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.adapter.get.AdapterInfo[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.Get',
                            adapter=adapter,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiNetworkportalIpconfig(Base):
    '''
    Operations that can be performed on iSCSI Network Portal (iSCSI vmknic)'s IP configuration
    '''
    moid = 'ha-cli-handler-iscsi-networkportal-ipconfig'
    def set(self, adapter, ip, subnet, dns1=None, dns2=None, gateway=None, nic=None):
        '''
        Set iSCSI network portal IP configuration
        :param adapter: string, The iSCSI adapter name.
        :param dns1: string, The iSCSI network portal primary DNS address
        :param dns2: string, The iSCSI network portal secondary DNS address
        :param gateway: string, The iSCSI network portal gateway address
        :param ip: string, The iSCSI network portal IP address
        :param nic: string, The iSCSI network portal (vmknic)
        :param subnet: string, The iSCSI network portal subnet mask
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.networkportal.ipconfig.Set',
                            adapter=adapter,
                            dns1=dns1,
                            dns2=dns2,
                            gateway=gateway,
                            ip=ip,
                            nic=nic,
                            subnet=subnet,
                            )
    def get(self, adapter, nic=None):
        '''
        Get iSCSI network portal ip configuration
        :param adapter: string, The iSCSI adapter name.
        :param nic: string, The iSCSI network portal (vmknic)
        :returns: vim.EsxCLI.iscsi.networkportal.ipconfig.get.NetworkPortal[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.networkportal.ipconfig.Get',
                            adapter=adapter,
                            nic=nic,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def StorageDrsIolbDisabledInternally(vim, *args, **kwargs):
    '''The fault occurs when Storage DRS disables IO Load balancing internally even
    though it is enabled by the user. This can happen due to one of the following
    reasons: 1. SIOC couldn't get enabled on at least one of the datastores 2. The
    connectivity between hosts and datastores is not uniform for all datastores. 3.
    Some statistics are not available to run IO load balancing'''

    obj = vim.client.factory.create('{urn:vim25}StorageDrsIolbDisabledInternally')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DistributedVirtualPortgroupMetaTagName = Enum(
    'dvsName',
    'portgroupName',
    'portIndex',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreAdapterStats(Base):
    '''
    Stats operations on SCSI Host Bus Adapters on the system.
    '''
    moid = 'ha-cli-handler-storage-core-adapter-stats'
    def get(self, adapter=None):
        '''
        List the SCSI stats for the SCSI Host Bus Adapters in the system.
        :param adapter: string, Limit the stats output to one adapter
        :returns: vim.EsxCLI.storage.core.adapter.stats.get.ScsiAdapterStats[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.adapter.stats.Get',
                            adapter=adapter,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PhysicalNicProfile(vim, *args, **kwargs):
    '''The PhysicalNicProfile data object represents physical NIC configuration. Use
    the policy list for access to configuration data for the physical NIC profile.
    Use the property list for access to subprofile configuration data, if any.'''

    obj = vim.client.factory.create('{urn:vim25}PhysicalNicProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'key', 'enabled' ]
    optional = [ 'policy', 'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ActionParameter = Enum(
    'alarm',
    'alarmName',
    'declaringSummary',
    'eventDescription',
    'newStatus',
    'oldStatus',
    'target',
    'targetName',
    'triggeringSummary',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostStorageSystem(ExtensibleManagedObject):
    '''This managed object gets and sets configuration information about the host's
    storage subsystem. The properties and operations are used to configure the host
    to make storage available for virtual machines. This object contains properties
    that are specific to ESX Server and general to both the ESX Server system and
    the hosted architecture.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostStorageSystem):
        super(HostStorageSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def fileSystemVolumeInfo(self):
        '''File system volume information for the host. See the FileSystemVolumeInfo data
        object type for more information.'''
        return self.update('fileSystemVolumeInfo')
    @property
    def multipathStateInfo(self):
        '''Runtime information about the state of a multipath path. A null value will be
        returned if path state information is not available for this system.'''
        return self.update('multipathStateInfo')
    @property
    def storageDeviceInfo(self):
        '''Host storage information up to the device level.'''
        return self.update('storageDeviceInfo')
    @property
    def systemFile(self):
        '''Datastore paths of files used by the host system on mounted volumes, for
        instance, the COS vmdk file of the host. For information on datastore paths,
        see Datastore.'''
        return self.update('systemFile')

    
    
    def AddInternetScsiSendTargets(self, iScsiHbaDevice, targets):
        '''Adds Send Target entries to the host bus adapter discovery list. The
        DiscoveryProperties.sendTargetsDiscoveryEnabled flag must be set to true.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targets: An array of iSCSI send targets.
        
        '''
        return self.delegate("AddInternetScsiSendTargets")(iScsiHbaDevice, targets)
    
    def AddInternetScsiStaticTargets(self, iScsiHbaDevice, targets):
        '''Adds Static Target entries to the host bus adapter discovery list. The
        DiscoveryProperty.staticTargetDiscoveryEnabled must be set to true.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targets: An array of iSCSI static targets to add.
        
        '''
        return self.delegate("AddInternetScsiStaticTargets")(iScsiHbaDevice, targets)
    
    def AttachScsiLun(self, lunUuid):
        '''Allow I/O issue to the specified detached ScsiLun. The ScsiLun is
        administratively configured on, if the attach operation is successful. See
        DetachScsiLun.Allow I/O issue to the specified detached ScsiLun. The ScsiLun is
        administratively configured on, if the attach operation is successful. See
        DetachScsiLun.
        
        :param lunUuid: The uuid of the ScsiLun to update.
        
        '''
        return self.delegate("AttachScsiLun")(lunUuid)
    
    def AttachVmfsExtent(self, vmfsPath, extent):
        '''Extends a VMFS by attaching a disk partition as an extent.
        
        :param vmfsPath: The path of the VMFS to extend. See FileSystemMountInfo.
        
        :param extent: A data object that describes the specification of a Disk partition.
        
        '''
        return self.delegate("AttachVmfsExtent")(vmfsPath, extent)
    
    def ComputeDiskPartitionInfo(self, devicePath, layout, partitionFormat=None):
        '''Computes the disk partition information given the desired disk layout. The
        server computes a new partition information object for a specific disk
        representing the desired layout.See HostDiskPartitionInfoPartitionFormat
        
        :param devicePath: The name of the device path for the specific disk.See HostDiskPartitionInfoPartitionFormat
        
        :param layout: A data object that describes the disk partition layout.See HostDiskPartitionInfoPartitionFormat
        
        :param partitionFormat: Specifies the desired partition format to be computed from the block range. If partitionFormat is not specified, the existing partitionFormat on disk is used, if the disk is not blank and mbr otherwise.See HostDiskPartitionInfoPartitionFormatvSphere API 5.0
        
        '''
        return self.delegate("ComputeDiskPartitionInfo")(devicePath, layout, partitionFormat)
    
    def ComputeDiskPartitionInfoForResize(self, partition, blockRange, partitionFormat=None):
        '''Computes the disk partition information for the purpose of resizing a given
        partition.See HostDiskPartitionInfoPartitionFormat
        
        :param partition: The disk partition to resize.See HostDiskPartitionInfoPartitionFormat
        
        :param blockRange: Specifies the desired block range for the resized partition. The start of the block range specified should match that of the current partition.See HostDiskPartitionInfoPartitionFormat
        
        :param partitionFormat: Specifies the desired partition format to be computed from the block range. If partitionFormat is not specified, the existing partitionFormat on disk is used, if the disk is not blank and mbr otherwise.See HostDiskPartitionInfoPartitionFormatvSphere API 5.0
        
        '''
        return self.delegate("ComputeDiskPartitionInfoForResize")(partition, blockRange, partitionFormat)
    
    def DetachScsiLun(self, lunUuid):
        '''Disallow I/O issue to the specified ScsiLun. The ScsiLun is detached, i.e.
        administratively configured off until a subsequent attachScsiLun is performed,
        if the operation is successful. See AttachScsiLun.Disallow I/O issue to the
        specified ScsiLun. The ScsiLun is detached, i.e. administratively configured
        off until a subsequent attachScsiLun is performed, if the operation is
        successful. See AttachScsiLun.
        
        :param lunUuid: The uuid of the ScsiLun device to detach.
        
        '''
        return self.delegate("DetachScsiLun")(lunUuid)
    
    def DisableMultipathPath(self, pathName):
        '''Disables an enabled path for a Logical Unit. Use the path name from
        HostMultipathStateInfoPath or HostMultipathInfoPath.
        
        :param pathName: The name of the path to disable.
        
        '''
        return self.delegate("DisableMultipathPath")(pathName)
    
    def DiscoverFcoeHbas(self, fcoeSpec):
        '''Initiates FCoE discovery using the given FcoeSpecification. Upon success,
        discovered VNPorts will have registered with the system as FCoE HBAs.
        
        :param fcoeSpec: 
        
        '''
        return self.delegate("DiscoverFcoeHbas")(fcoeSpec)
    
    def EnableMultipathPath(self, pathName):
        '''Enables a disabled path for a Logical Unit. Use the path name from
        HostMultipathStateInfoPath or HostMultipathInfoPath.
        
        :param pathName: The name of the path to enable.
        
        '''
        return self.delegate("EnableMultipathPath")(pathName)
    
    def ExpandVmfsExtent(self, vmfsPath, extent):
        '''Expands a VMFS extent as specified by the Disk partition specification.
        
        :param vmfsPath: The path of the VMFS to expand. See FileSystemMountInfo.
        
        :param extent: The disk partition corresponding to the extent to be expanded.
        
        '''
        return self.delegate("ExpandVmfsExtent")(vmfsPath, extent)
    
    def FormatVmfs(self, createSpec):
        '''Formats a new VMFS on a disk partition.
        
        :param createSpec: A data object that describes the VMware File System (VMFS) creation specification.
        
        '''
        return self.delegate("FormatVmfs")(createSpec)
    
    def MarkForRemoval(self, hbaName, remove):
        '''Mark or unmark the given FCoE HBA for removal from the host system. Marking an
        FCoE HBA for removal will result in the HBA not being discovered upon host
        reboot. Until reboot, the HBA remains visible in the storage topology.
        
        :param hbaName: 
        
        :param remove: 
        
        '''
        return self.delegate("MarkForRemoval")(hbaName, remove)
    
    def MountVmfsVolume(self, vmfsUuid):
        '''Mount the unmounted Vmfs volume. A newly discovered vmfs volume will be mounted
        unless, it has been explicitly unmounted. The default mount behavior of Vmfs
        volumes is auto-mount. See UnmountVmfsVolume.Mount the unmounted Vmfs volume. A
        newly discovered vmfs volume will be mounted unless, it has been explicitly
        unmounted. The default mount behavior of Vmfs volumes is auto-mount. See
        UnmountVmfsVolume.
        
        :param vmfsUuid: 
        
        '''
        return self.delegate("MountVmfsVolume")(vmfsUuid)
    
    def QueryPathSelectionPolicyOptions(self):
        '''Queries the set of path selection policy options. The set of policy options
        indicates what path selection policies can be used by a device managed by
        native multipathing. Devices managed through native multipathing are described
        in the HostMultipathInfo data object.Queries the set of path selection policy
        options. The set of policy options indicates what path selection policies can
        be used by a device managed by native multipathing. Devices managed through
        native multipathing are described in the HostMultipathInfo data object.
        
        '''
        return self.delegate("QueryPathSelectionPolicyOptions")()
    
    def QueryStorageArrayTypePolicyOptions(self):
        '''Queries the set of storage array type policy options. The set of policy options
        indicates what storage array type policies can be used by a device managed by
        native multipathing. Devices managed through native multipathing are described
        in the HostMultipathInfo data object.Queries the set of storage array type
        policy options. The set of policy options indicates what storage array type
        policies can be used by a device managed by native multipathing. Devices
        managed through native multipathing are described in the HostMultipathInfo data
        object.
        
        '''
        return self.delegate("QueryStorageArrayTypePolicyOptions")()
    
    def QueryUnresolvedVmfsVolume(self):
        '''Get the list of unbound VMFS volumes. For sharing a volume across hosts, a VMFS
        volume is bound to its underlying block device storage. When a low level block
        copy is performed to copy or move the VMFS volume, the copied volume will be
        unbound.
        
        '''
        return self.delegate("QueryUnresolvedVmfsVolume")()
    
    def RefreshStorageSystem(self):
        '''Obtains the latest host storage information related to storage devices,
        topology, and file systems. The ESX host updates its storage information
        asynchronously. The RefreshStorageSystem method obtains information from the
        host platform and it performs a rescan of VMFS volumes. It does not look for
        new devices.Obtains the latest host storage information related to storage
        devices, topology, and file systems. The ESX host updates its storage
        information asynchronously. The RefreshStorageSystem method obtains information
        from the host platform and it performs a rescan of VMFS volumes. It does not
        look for new devices.Obtains the latest host storage information related to
        storage devices, topology, and file systems. The ESX host updates its storage
        information asynchronously. The RefreshStorageSystem method obtains information
        from the host platform and it performs a rescan of VMFS volumes. It does not
        look for new devices.Obtains the latest host storage information related to
        storage devices, topology, and file systems. The ESX host updates its storage
        information asynchronously. The RefreshStorageSystem method obtains information
        from the host platform and it performs a rescan of VMFS volumes. It does not
        look for new devices.
        
        '''
        return self.delegate("RefreshStorageSystem")()
    
    def RemoveInternetScsiSendTargets(self, iScsiHbaDevice, targets):
        '''Removes Send Target entries from the host bus adapter discovery list. The
        DiscoveryProperty.sendTargetsDiscoveryEnabled must be set to true. If any of
        the targets provided as parameters are not found in the existing list, the
        other targets are removed and an exception is thrown.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targets: An array of iSCSI send targets to remove.
        
        '''
        return self.delegate("RemoveInternetScsiSendTargets")(iScsiHbaDevice, targets)
    
    def RemoveInternetScsiStaticTargets(self, iScsiHbaDevice, targets):
        '''Removes static target entries from the host bus adapter discovery list. The
        DiscoveryProperty.staticTargetDiscoveryEnabled must be set to true. If any of
        the targets provided as parameters are not found in the existing list, the
        other targets are removed and an exception is thrown.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targets: An array of iSCSI static targets to remove.
        
        '''
        return self.delegate("RemoveInternetScsiStaticTargets")(iScsiHbaDevice, targets)
    
    def RescanAllHba(self):
        '''Scans all host bus adapters to obtain the current list of devices and device
        topology. The RescanAllHba method looks for new devices, removed devices, and
        path changes.Scans all host bus adapters to obtain the current list of devices
        and device topology. The RescanAllHba method looks for new devices, removed
        devices, and path changes.Scans all host bus adapters to obtain the current
        list of devices and device topology. The RescanAllHba method looks for new
        devices, removed devices, and path changes.Scans all host bus adapters to
        obtain the current list of devices and device topology. The RescanAllHba method
        looks for new devices, removed devices, and path changes.
        
        '''
        return self.delegate("RescanAllHba")()
    
    def RescanHba(self, hbaDevice):
        '''Issues a request to rescan a specific host bus adapter for new storage devices.
        
        :param hbaDevice: The device of the host bus adapter.
        
        '''
        return self.delegate("RescanHba")(hbaDevice)
    
    def RescanVmfs(self):
        '''Rescans for new Virtual Machine File Systems (VMFS). The RefreshStorageSystem
        method also performs a VMFS rescan.Rescans for new Virtual Machine File Systems
        (VMFS). The RefreshStorageSystem method also performs a VMFS rescan.
        
        '''
        return self.delegate("RescanVmfs")()
    
    def ResolveMultipleUnresolvedVmfsVolumes(self, resolutionSpec):
        '''Resignature or 'Force Mount' list of unbound VMFS volumes. To safely enable
        sharing of the volume across hosts, a VMFS volume is bound to its underlying
        block device storage. When a low level block copy is performed to copy or move
        the VMFS volume, the copied volume will be unbound. In order for the VMFS
        volume to be usable, a resolution operation is needed to determine whether the
        VMFS volume should be treated as a new volume or not and what extents compose
        that volume in the event there is more than one unbound volume.Resignature or
        'Force Mount' list of unbound VMFS volumes. To safely enable sharing of the
        volume across hosts, a VMFS volume is bound to its underlying block device
        storage. When a low level block copy is performed to copy or move the VMFS
        volume, the copied volume will be unbound. In order for the VMFS volume to be
        usable, a resolution operation is needed to determine whether the VMFS volume
        should be treated as a new volume or not and what extents compose that volume
        in the event there is more than one unbound volume.Resignature or 'Force Mount'
        list of unbound VMFS volumes. To safely enable sharing of the volume across
        hosts, a VMFS volume is bound to its underlying block device storage. When a
        low level block copy is performed to copy or move the VMFS volume, the copied
        volume will be unbound. In order for the VMFS volume to be usable, a resolution
        operation is needed to determine whether the VMFS volume should be treated as a
        new volume or not and what extents compose that volume in the event there is
        more than one unbound volume.
        
        :param resolutionSpec: List of data object that describes what the disk extents to be used for creating the new VMFS volume.
        
        '''
        return self.delegate("ResolveMultipleUnresolvedVmfsVolumes")(resolutionSpec)
    
    def RetrieveDiskPartitionInfo(self, devicePath):
        '''Gets the partition information for the disks named by the device names.
        
        :param devicePath: An array of device path names that identify disks. See ScsiDisk.
        
        '''
        return self.delegate("RetrieveDiskPartitionInfo")(devicePath)
    
    def SetMultipathLunPolicy(self, lunId, policy):
        '''Updates the path selection policy for a Logical Unit. Use the LUN uuid from
        HostMultipathInfoLogicalUnit.
        
        :param lunId: The logical unit ID
        
        :param policy: A data object that describes a path selection policy for the logical unit.
        
        '''
        return self.delegate("SetMultipathLunPolicy")(lunId, policy)
    
    def UnmountForceMountedVmfsVolume(self, vmfsUuid):
        '''Unmount the 'forceMounted' Vmfs volume. When a low level block copy is
        performed to copy or move the VMFS volume, the copied volume is unresolved. For
        the VMFS volume to be usable, a resolution operation is applied. As part of
        resolution operation, user may decide to keep the original VMFS Uuid. Once the
        resolution is applied, the VMFS volume is mounted on the host for its use. User
        can unmount the VMFS volume if it is not being used by any registered VMs.
        
        :param vmfsUuid: 
        
        '''
        return self.delegate("UnmountForceMountedVmfsVolume")(vmfsUuid)
    
    def UnmountVmfsVolume(self, vmfsUuid):
        '''Unmount the Vmfs volume. An unmounted volume cannot be used for any filesystem
        operation requiring I/O. In contrast to removal, this operation does not
        destroy or alter partitions on which vmfs volumes reside. The mountState will
        be persisted across filesystem rescans and host reboots. See
        MountVmfsVolume.Unmount the Vmfs volume. An unmounted volume cannot be used for
        any filesystem operation requiring I/O. In contrast to removal, this operation
        does not destroy or alter partitions on which vmfs volumes reside. The
        mountState will be persisted across filesystem rescans and host reboots. See
        MountVmfsVolume.
        
        :param vmfsUuid: 
        
        '''
        return self.delegate("UnmountVmfsVolume")(vmfsUuid)
    
    def UpdateDiskPartitions(self, devicePath, spec):
        '''Changes the partitions on the disk by supplying a partition specification and
        the device name.
        
        :param devicePath: The name of the device path for the specific disk.
        
        :param spec: A data object that describes the disk partition table specification used to configure the partitions on a disk.
        
        '''
        return self.delegate("UpdateDiskPartitions")(devicePath, spec)
    
    def UpdateInternetScsiAdvancedOptions(self, iScsiHbaDevice, options, targetSet=None):
        '''Updates the advanced options the iSCSI host bus adapter or the discovery
        addresses and targets associated with it.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targetSet: The set the targets to configure. If not provided, the settings will be applied to the host bus adapter itself.
        
        :param options: The list of options to set.
        
        '''
        return self.delegate("UpdateInternetScsiAdvancedOptions")(iScsiHbaDevice, targetSet, options)
    
    def UpdateInternetScsiAlias(self, iScsiHbaDevice, iScsiAlias):
        '''Updates the alias of an iSCSI host bus adapter.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param iScsiAlias: The new value for the alias of the adapter.
        
        '''
        return self.delegate("UpdateInternetScsiAlias")(iScsiHbaDevice, iScsiAlias)
    
    def UpdateInternetScsiAuthenticationProperties(self, iScsiHbaDevice, authenticationProperties, targetSet=None):
        '''Updates the authentication properties for one or more targets or discovery
        addresses associated with an iSCSI host bus adapter.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter. associated with the target.
        
        :param authenticationProperties: The data object that represents the authentication settings to set.
        
        :param targetSet: The set the targets to configure. Optional, when obmitted will configura the authentication properties for the adapter instead.vSphere API 4.0
        
        '''
        return self.delegate("UpdateInternetScsiAuthenticationProperties")(iScsiHbaDevice, authenticationProperties, targetSet)
    
    def UpdateInternetScsiDigestProperties(self, iScsiHbaDevice, digestProperties, targetSet=None):
        '''Updates the digest properties for the iSCSI host bus adapter or the discovery
        addresses and targets associated with it.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param targetSet: The set the targets to configure. If not provided, the settings will be applied to the host bus adapter itself.
        
        :param digestProperties: The data object that represents the digest settings to set.
        
        '''
        return self.delegate("UpdateInternetScsiDigestProperties")(iScsiHbaDevice, targetSet, digestProperties)
    
    def UpdateInternetScsiDiscoveryProperties(self, iScsiHbaDevice, discoveryProperties):
        '''Updates the Discovery properties for an iSCSI host bus adapter.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param discoveryProperties: The discovery settings for this host bus adapter.
        
        '''
        return self.delegate("UpdateInternetScsiDiscoveryProperties")(iScsiHbaDevice, discoveryProperties)
    
    def UpdateInternetScsiIPProperties(self, iScsiHbaDevice, ipProperties):
        '''Updates the IP properties for an iSCSI host bus adapter.
        
        :param iScsiHbaDevice: The device of the Internet SCSI HBA adapter.
        
        :param ipProperties: A data object representing the IP properties for the host bus adapter
        
        '''
        return self.delegate("UpdateInternetScsiIPProperties")(iScsiHbaDevice, ipProperties)
    
    def UpdateInternetScsiName(self, iScsiHbaDevice, iScsiName):
        '''Updates the name of an iSCSI host bus adapter.
        
        :param iScsiHbaDevice: The current name of the Internet SCSI HBA adapter.
        
        :param iScsiName: The new name for the Internet SCSI HBA adapter
        
        '''
        return self.delegate("UpdateInternetScsiName")(iScsiHbaDevice, iScsiName)
    
    def UpdateScsiLunDisplayName(self, lunUuid, displayName):
        '''Update the mutable display name associated with a ScsiLun. The ScsiLun to be
        updated is identified using the specified uuid.
        
        :param lunUuid: The uuid of the ScsiLun to update.
        
        :param displayName: The new name to assign to the ScsiLun object.
        
        '''
        return self.delegate("UpdateScsiLunDisplayName")(lunUuid, displayName)
    
    def UpdateSoftwareInternetScsiEnabled(self, enabled):
        '''Enables or disables Software iSCSI.
        
        :param enabled: True to enable the iSCSI; false to disable it
        
        '''
        return self.delegate("UpdateSoftwareInternetScsiEnabled")(enabled)
    
    def UpgradeVmfs(self, vmfsPath):
        '''Upgrades the VMFS to the latest supported VMFS version.Upgrades the VMFS to the
        latest supported VMFS version.
        
        :param vmfsPath: The path of the VMFS.
        
        '''
        return self.delegate("UpgradeVmfs")(vmfsPath)
    
    def UpgradeVmLayout(self):
        '''Iterates over all registered virtual machines. For each VM which .vmx file is
        located on the service console and all disks are available on VMFS3 or NAS, it
        will relocate the disks into directories if stored in the ROOT, and relocate
        the VMX file into the directory too. Events are logged for each virtual machine
        that is relocated.Iterates over all registered virtual machines. For each VM
        which .vmx file is located on the service console and all disks are available
        on VMFS3 or NAS, it will relocate the disks into directories if stored in the
        ROOT, and relocate the VMX file into the directory too. Events are logged for
        each virtual machine that is relocated.
        
        '''
        return self.delegate("UpgradeVmLayout")()



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def FileQuery(vim, *args, **kwargs):
    '''The data object type that describes the base query specification. Contains
    query filters and details that apply to every file. Querying only file details
    generally does not require opening files and so is an efficient query. Derived
    types add query parameters specific to the type of file.'''

    obj = vim.client.factory.create('{urn:vim25}FileQuery')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ResourcePlanningManager(BaseEntity):
    ''''''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ResourcePlanningManager):
        super(ResourcePlanningManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def EstimateDatabaseSize(self, dbSizeParam):
        '''Estimates the database size required to store VirtualCenter data.
        
        :param dbSizeParam: DatabaseSizeParam Contains the summary of an inventory for which the database size requirements are to be computed. It also contains the historic interval setting for which the database computations are to be done. This is an optional parameter and the current virtual center historical settings are used by default. There are many other optional fields in the dbSizeParam structure that are appropriately filled up based on some heuristics.
        
        '''
        return self.delegate("EstimateDatabaseSize")(dbSizeParam)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostApplyProfile(vim, *args, **kwargs):
    '''The HostApplyProfile data object provides access to subprofiles that contain
    configuration data for different host capabilities. The Profile Engine will use
    any configuration data that you supply to overwrite the host configuration. See
    the ExecuteHostProfile and ApplyHostConfig_Task methods.'''

    obj = vim.client.factory.create('{urn:vim25}HostApplyProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'enabled' ]
    optional = [ 'authentication', 'datetime', 'firewall', 'memory', 'network', 'option',
        'security', 'service', 'storage', 'userAccount', 'usergroupAccount', 'policy',
        'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemStatsUptime(Base):
    '''
    System uptime, in microseconds
    '''
    moid = 'ha-cli-handler-system-stats-uptime'
    def get(self):
        '''
        Disply the number of microseconds the system has been running.
        :returns: long
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.stats.uptime.Get',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterDasConfigInfoVmMonitoringState = Enum(
    'vmAndAppMonitoring',
    'vmMonitoringDisabled',
    'vmMonitoringOnly',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def TooManyConsecutiveOverrides(vim, *args, **kwargs):
    '''Thrown if there are too many consecutive user overrides of server-managed
    settings. There are some properties which users are allowed to set only in
    certain configurations. The system controls them in other configurations, and
    overrides all user updates. If there are such frequent user overrides in the
    system controlled state that the next update arrives before the previous
    override is complete, the system can enter a tight loop and appear unresponsive
    to the user. This would typically happen if the user overrides are generated by
    a script. If the number of such user overrides exceeds the supported maximum,
    we conceptually "throw" this fault. In practice, this is converted into a host-
    level ConfigIssue in VC.'''

    obj = vim.client.factory.create('{urn:vim25}TooManyConsecutiveOverrides')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class HardwareBootdevice(Base):
    '''
    Boot device information.
    '''
    moid = 'ha-cli-handler-hardware-bootdevice'
    def list(self):
        '''
        List the boot device order, if available, for this host.
        :returns: vim.EsxCLI.hardware.bootdevice.list.BootDeviceList[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.hardware.bootdevice.List',
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiSession(Base):
    '''
    Operations that can be performed on iSCSI sessions
    '''
    moid = 'ha-cli-handler-iscsi-session'
    def add(self, adapter, isid=None, name=None):
        '''
        Login sessions on current iSCSI configuration.
        :param adapter: string, The iSCSI adapter name.
        :param isid: string, The isid of a session to duplicate for login.
        :param name: string, The iSCSI target name.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.session.Add',
                            adapter=adapter,
                            isid=isid,
                            name=name,
                            )
    def list(self, adapter=None, isid=None, name=None):
        '''
        List iSCSI Sessions.
        :param adapter: string, The iSCSI adapter name.
        :param isid: string, The iSCSI session identifier.
        :param name: string, The iSCSI target name.
        :returns: vim.EsxCLI.iscsi.session.list.Session[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.session.List',
                            adapter=adapter,
                            isid=isid,
                            name=name,
                            )
    def remove(self, adapter, isid=None, name=None):
        '''
        Logout sessions on current iSCSI configuration.
        :param adapter: string, The iSCSI adapter name.
        :param isid: string, The isid of a session to duplicate for login.
        :param name: string, The name of the target to login to.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.session.Remove',
                            adapter=adapter,
                            isid=isid,
                            name=name,
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class GuestFileManager(BaseEntity):
    '''FileManager is the managed object that provides APIs to manipulate the guest
    operating system files.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.GuestFileManager):
        super(GuestFileManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ChangeFileAttributesInGuest(self, vm, auth, guestFilePath, fileAttributes):
        '''Changes the file attributes of a specified file inside the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param guestFilePath: The complete path to the file to be copied in the guest. If the file points to an symbolic link, then the attributes of the target file are changed.
        
        :param fileAttributes: Specifies the different file attributes of the guest file to be changed. See GuestFileAttributes. If any property is not specified, then the specific attribute of the file will be unchanged.
        
        '''
        return self.delegate("ChangeFileAttributesInGuest")(vm, auth, guestFilePath, fileAttributes)
    
    def CreateTemporaryDirectoryInGuest(self, vm, auth, prefix, suffix, directoryPath=None):
        '''Creates a temporary directory.Creates a temporary directory.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param prefix: The prefix to be given to the new temporary directory.
        
        :param suffix: The suffix to be given to the new temporary directory.
        
        :param directoryPath: The complete path to the directory in which to create the new directory. If unset or an empty string, a guest-specific location will be used.
        
        '''
        return self.delegate("CreateTemporaryDirectoryInGuest")(vm, auth, prefix, suffix, directoryPath)
    
    def CreateTemporaryFileInGuest(self, vm, auth, prefix, suffix, directoryPath=None):
        '''Creates a temporary file.Creates a temporary file.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param prefix: The prefix to be given to the new temporary file.
        
        :param suffix: The suffix to be given to the new temporary file.
        
        :param directoryPath: The complete path to the directory in which to create the file. If unset, or an empty string, a guest-specific location will be used.
        
        '''
        return self.delegate("CreateTemporaryFileInGuest")(vm, auth, prefix, suffix, directoryPath)
    
    def DeleteDirectoryInGuest(self, vm, auth, directoryPath, recursive):
        '''Deletes a directory in the guest OS.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param directoryPath: The complete path to the directory to be deleted.
        
        :param recursive: If true, all subdirectories are also deleted. If false, the directory must be empty for the operation to succeed.
        
        '''
        return self.delegate("DeleteDirectoryInGuest")(vm, auth, directoryPath, recursive)
    
    def DeleteFileInGuest(self, vm, auth, filePath):
        '''Deletes a file in the guest OS
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param filePath: The complete path to the file or symbolic link to be deleted.
        
        '''
        return self.delegate("DeleteFileInGuest")(vm, auth, filePath)
    
    def InitiateFileTransferFromGuest(self, vm, auth, guestFilePath):
        '''Initiates an operation to transfer a file from the guest.Initiates an operation
        to transfer a file from the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data.
        
        :param guestFilePath: The complete path to the file inside the guest that has to be transferred to the client. It cannot be a path to a directory or a symbolic link.
        
        '''
        return self.delegate("InitiateFileTransferFromGuest")(vm, auth, guestFilePath)
    
    def InitiateFileTransferToGuest(self, vm, auth, guestFilePath, fileAttributes, fileSize, overwrite):
        '''Initiates an operation to transfer a file to the guest.Initiates an operation
        to transfer a file to the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param guestFilePath: The complete destination path in the guest to transfer the file from the client. It cannot be a path to a directory or a symbolic link.
        
        :param fileAttributes: File attributes of the file that has to be created in the guest. See GuestFileAttributes. If any file attribute is not specified, then the default value of that property will be set for the file.
        
        :param fileSize: Size of the file to transfer to the guest in bytes.
        
        :param overwrite: If set, the destination file is clobbered.
        
        '''
        return self.delegate("InitiateFileTransferToGuest")(vm, auth, guestFilePath, fileAttributes, fileSize, overwrite)
    
    def ListFilesInGuest(self, vm, auth, filePath, index=None, maxResults=None, matchPattern=None):
        '''Returns information about files or directories in the guest.Returns information
        about files or directories in the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param filePath: The complete path to the directory or file to query.
        
        :param index: Which result to start the list with. The default is 0.
        
        :param maxResults: The maximum number of results to return. The default is 50.
        
        :param matchPattern: A filter for the return values. Match patterns are specified using perl-compatible regular expressions. If matchPattern is unset, then the pattern '.*' is used.
        
        '''
        return self.delegate("ListFilesInGuest")(vm, auth, filePath, index, maxResults, matchPattern)
    
    def MakeDirectoryInGuest(self, vm, auth, directoryPath, createParentDirectories):
        '''Creates a directory in the guest OS
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param directoryPath: The complete path to the directory to be created.
        
        :param createParentDirectories: Whether any parent directories are to be created.
        
        '''
        return self.delegate("MakeDirectoryInGuest")(vm, auth, directoryPath, createParentDirectories)
    
    def MoveDirectoryInGuest(self, vm, auth, srcDirectoryPath, dstDirectoryPath):
        '''Moves or renames a directory in the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param srcDirectoryPath: The complete path to the directory to be moved.
        
        :param dstDirectoryPath: The complete path to the where the directory is moved or its new name. It cannot be a path to an existing directory or an existing file.
        
        '''
        return self.delegate("MoveDirectoryInGuest")(vm, auth, srcDirectoryPath, dstDirectoryPath)
    
    def MoveFileInGuest(self, vm, auth, srcFilePath, dstFilePath, overwrite):
        '''Renames a file in the guest.
        
        :param vm: Virtual Machine to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        :param srcFilePath: The complete path to the original file or symbolic link to be moved.
        
        :param dstFilePath: The complete path to the where the file is renamed. It cannot be a path to an existing diectory.
        
        :param overwrite: If set, the destination file is clobbered.
        
        '''
        return self.delegate("MoveFileInGuest")(vm, auth, srcFilePath, dstFilePath, overwrite)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class OvfManager(BaseEntity):
    '''Service interface to parse and generate OVF descriptors.The purpose of this
    interface is to make it easier for callers to import VMs and vApps from OVF
    packages and to export VI packages to OVF. In the following description, the
    term "client" is used to mean any caller of the interface.This interface only
    converts between OVF and VI types. To actually import and export entities, use
    ResourcePool.importVApp, VirtualMachine.exportVm and VirtualApp.exportVApp.For
    the import scenario, the typical sequence of events is as follows:The client
    calls parseDescriptor to obtain information about the OVF descriptor. This
    typically includes information (such as a list of networks) that must be mapped
    to VI infrastructure entities.The OVF descriptor is validated against the OVF
    Specification, and any errors or warnings are returned as part of the
    ParseResult. For example, the parser might encounter a section marked required
    that it does not understand, or the XML descriptor might be malformed.The
    client decides on network mappings, datastore, properties etc. It then calls
    createImportSpec to obtain the parameters needed to call
    ResourcePool.importVApp.If any warnings are present, the client will review
    these and decide whether to proceed or not. If errors are present, the
    ImportSpec will be missing, so the client is forced to give up or fix the
    problems and then try again.The client now calls ResourcePool.importVApp,
    passing the ImportSpec as a parameter. This will create the virtual machines
    and VirtualApp objects in VI and return locations to which the files of the
    entity can be uploaded. It also returns a lease that controls the duration of
    the lock taken on the newly created inventory objects. When all files have been
    uploaded, the client must release this lease.Creating the OVF descriptor is the
    last part of exporting an entity to OVF. At this point, the client has already
    downloaded all files for the entity, optionally compressing and/or chunking
    them (however, the client may do a "dry run" of creating the descriptor before
    downloading the files. See OvfManager.createDescriptor).In addition to the
    entity reference itself, information about the choices made on these files is
    passed to createDescriptor as a list of OvfFile instances.The client must
    inspect and act upon warnings and errors as previously described.No matter if
    the export succeeds or fails, the client is responsible for releasing the
    shared state lock taken on the entity (by VirtualMaching.exportVm or
    VirtualApp.exportVApp) during the export.All result types contain warning and
    error lists. Warnings do not cause processing to fail, but the caller
    (typically, the user of a GUI client) may choose to reject the result based on
    the warnings issued.Errors cause processing to abort by definition.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.OvfManager):
        super(OvfManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CreateDescriptor(self, obj, cdp):
        '''Create an OVF descriptor for the specified ManagedEntity, which may be a
        VirtualMachine or a VirtualApp.Create an OVF descriptor for the specified
        ManagedEntity, which may be a VirtualMachine or a VirtualApp.Create an OVF
        descriptor for the specified ManagedEntity, which may be a VirtualMachine or a
        VirtualApp.Create an OVF descriptor for the specified ManagedEntity, which may
        be a VirtualMachine or a VirtualApp.Create an OVF descriptor for the specified
        ManagedEntity, which may be a VirtualMachine or a VirtualApp.
        
        :param obj: The entity to export. Supported types are VirtualMachine and VirtualApp.
        
        :param cdp: Parameters to the method, bundled in an instance of CreateDescriptorParams.
        
        '''
        return self.delegate("CreateDescriptor")(obj, cdp)
    
    def CreateImportSpec(self, ovfDescriptor, resourcePool, datastore, cisp):
        '''Validate the OVF descriptor against the hardware supported by the host system.
        If the validation succeeds, return a result containing:Validate the OVF
        descriptor against the hardware supported by the host system. If the validation
        succeeds, return a result containing:
        
        :param ovfDescriptor: The OVF descriptor of the entity.
        
        :param resourcePool: The resource pool to import the entity to. May be a vApp.
        
        :param datastore: The datastore on which to create the inventory objects of the entity, for example "storage1". The privilege Datastore.AllocateSpace is required on the datastore.
        
        :param cisp: Additional parameters to the method, bundled in an instance of CreateImportSpecParams.
        
        '''
        return self.delegate("CreateImportSpec")(ovfDescriptor, resourcePool, datastore, cisp)
    
    def ParseDescriptor(self, ovfDescriptor, pdp):
        '''Parse the OVF descriptor and return as much information about it as possible
        without knowing the host on which it will be imported.Parse the OVF descriptor
        and return as much information about it as possible without knowing the host on
        which it will be imported.
        
        :param ovfDescriptor: The OVF descriptor to examine.
        
        :param pdp: Additional parameters for parseDescriptor, wrapped in an instance of ParseDescriptorParams.
        
        '''
        return self.delegate("ParseDescriptor")(ovfDescriptor, pdp)
    
    def ValidateHost(self, ovfDescriptor, host, vhp):
        '''Validate that the given OVF can be imported on the host.Validate that the given
        OVF can be imported on the host.
        
        :param ovfDescriptor: The OVF descriptor to examine.
        
        :param host: The host to validate against.
        
        :param vhp: Additional parameters for validateHost, wrapped in a ValidateHostParams instance.
        
        '''
        return self.delegate("ValidateHost")(ovfDescriptor, host, vhp)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostSystemIdentificationInfoIdentifier = Enum(
    'AssetTag',
    'OemSpecificString',
    'ServiceTag',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def InheritablePolicy(vim, *args, **kwargs):
    '''The base class for any type of setting or configuration that may get a
    inherited value.When used in a reconfgure operation specification, if inherited
    is true, it specifies the intention to change the values of subclass's
    properties to the inhertied values from the level above. In this case, users
    don't need to specify the values and any set property in the subclass will be
    ignored. if inherited is false, it specifies the intension to explicitly set
    subclass's properties to user specified values. Users should set the properties
    in the subclass with the desired values.When used in a configuration
    information object, The values of the properties in the subclass are the
    effective values. if inherited is true, the object is getting the effective
    values from upper level. If false, the values are explicitly set by a user.'''

    obj = vim.client.factory.create('{urn:vim25}InheritablePolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'inherited' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCorePathStats(Base):
    '''
    Stats operations pertaining to the pluggable storage architectures' device paths on the system. 
    '''
    moid = 'ha-cli-handler-storage-core-path-stats'
    def get(self, path=None):
        '''
        List the SCSI stats for the SCSI Paths in the system.
        :param path: string, Limit the stats output to one specific path. This path name can be the runtime name or the path UID.
        :returns: vim.EsxCLI.storage.core.path.stats.get.ScsiPathStats[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.path.stats.Get',
                            path=path,
                            )   


from json import loads, dumps
from logging import getLogger
from infi.pyutils.lazy import cached_method, clear_cache
from pyvisdk import __version__

logger = getLogger(__name__)

from .errors import ExtensionAlreadyRegisteredException, ExtensionIsFullException, ExtensionNotRegisteredException
class TaskExtension(object):
    DESCRIPTION = "PyVISDK task facade extension"
    VERSION = __version__
    MAX_LENGTH = 255

    def __init__(self, client, key):
        super(TaskExtension, self).__init__()
        self._client = client
        self._managed_object = self._client.service_content.extensionManager
        self._key = key

    @cached_method
    def is_registered(self):
        return len(self._get_extensions_data_object()) > 0

    def register(self):
        if self.is_registered():
            raise ExtensionAlreadyRegisteredException()
        extension = self._new_extension()
        self._managed_object.RegisterExtension(extension)
        clear_cache(self)

    def is_task_registered(self, expected):
        return any([actual == expected for _, actual in self._get_tasks()])

    def get_task_id(self, task_name):
        if not self.is_registered():
            raise ExtensionNotRegisteredException()
        if not self.is_task_registered(task_name):
            raise KeyError(task_name)
        for task_tuple in self._get_tasks():
            if task_tuple[1] == task_name:
                return task_tuple[0]

    def add_task(self, task_name):
        task_id = str(hash(task_name))
        self._set_tasks([(task_id, task_name)] + self._get_tasks())
        clear_cache(self)

    @cached_method
    def _get_extensions_data_object(self):
        return [extension for extension in self._managed_object.extensionList
                if extension.key == self._key]

    @cached_method
    def _get_tasks(self):
        data_object = self._get_extensions_data_object()[0]
        return loads(data_object.description.summary or '[]')

    def _set_tasks(self, tasks):
        extension = self._get_extensions_data_object()[0]
        extension.description.summary = dumps(tasks)
        if len(extension.description.summary) > self.MAX_LENGTH:
            raise ExtensionIsFullException
        extension.taskList = [self._new_task_type_info(task_id) for task_id, _ in tasks]
        extension.resourceList = self._new_task_extension_resource_info(tasks)
        self._managed_object.UpdateExtension(extension)
        clear_cache(self)

    def _new_extension(self):
        from pyvisdk.do.extension import Extension
        from pyvisdk.do.description import Description
        description = Description(self._client, label=self.DESCRIPTION, summary='')
        return Extension(self._client, description=description, key=self._key,
                         lastHeartbeatTime=self._get_heartbeat(),
                         version=self.VERSION)

    def _get_heartbeat(self):
        return self._client.service_instance.CurrentTime()

    def _new_task_type_info(self, task_id):
        from pyvisdk.do.extension_task_type_info import ExtensionTaskTypeInfo
        return ExtensionTaskTypeInfo(self._client, task_id)

    def _new_task_extension_resource_info(self, tasks):
        from pyvisdk.do.extension_resource_info import ExtensionResourceInfo
        from pyvisdk.do.key_value import KeyValue
        data = [KeyValue(self._client, "{}.label".format(task_id), task_name) for task_id, task_name in tasks]
        return ExtensionResourceInfo(self._client, locale="en", module="task", data=data)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HttpNfcLeaseState = Enum(
    'done',
    'error',
    'initializing',
    'ready',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

WillLoseHAProtectionResolution = Enum(
    'relocate',
    'svmotion',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.managed_entity import ManagedEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class Datastore(ManagedEntity):
    '''Represents a storage location for virtual machine files. A storage location can
    be a VMFS volume, a directory on Network Attached Storage, or a local file
    system path.A datastore is platform-independent and host-independent.
    Therefore, datastores do not change when the virtual machines they contain are
    moved between hosts. The scope of a datastore is a datacenter; the datastore is
    uniquely named within the datacenter.Any reference to a virtual machine or file
    accessed by any host within the datacenter must use a datastore path. A
    datastore path has the form "[<datastore>] <path>", where <datastore> is the
    datastore name, and <path> is a slash-delimited path from the root of the
    datastore. An example datastore path is "[storage] path/to/config.vmx".You may
    use the following characters in a path, but not in a datastore name: slash (/),
    backslash (\), and percent (%).All references to files in the VIM API are
    implicitly done using datastore paths.When a client creates a virtual machine,
    it may specify the name of the datastore, omitting the path; the system,
    meaning VirtualCenter or the host, automatically assigns filenames and creates
    directories on the given datastore. For example, specifying My_Datastore as a
    location for a virtual machine called MyVm results in a datastore location of
    My_Datastore\MyVm\MyVm.vmx.Datastores are configured per host. As part of host
    configuration, a HostSystem can be configured to mount a set of network drives.
    Multiple hosts may be configured to point to the same storage location. There
    exists only one Datastore object per Datacenter, for each such shared location.
    Each Datastore object keeps a reference to the set of hosts that have mounted
    the datastore. A Datastore object can be removed only if no hosts currently
    have the datastore mounted.Thus, managing datastores is done both at the host
    level and the datacenter level. Each host is configured explicitly with the set
    of datastores it can access. At the datacenter, a view of the datastores across
    the datacenter is shown.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.Datastore):
        super(Datastore, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def browser(self):
        '''DatastoreBrowser used to browse this datastore.'''
        return self.update('browser')
    @property
    def capability(self):
        '''Capabilities of this datastore.'''
        return self.update('capability')
    @property
    def host(self):
        '''Hosts attached to this datastore.'''
        return self.update('host')
    @property
    def info(self):
        '''Specific information about the datastore.'''
        return self.update('info')
    @property
    def iormConfiguration(self):
        '''Configuration of storage I/O resource management for the datastore. Currently
        we only support storage I/O resource management on VMFS volumes of a datastore.'''
        return self.update('iormConfiguration')
    @property
    def summary(self):
        '''Global properties of the datastore.'''
        return self.update('summary')
    @property
    def vm(self):
        '''Virtual machines stored on this datastore.'''
        return self.update('vm')

    
    
    def DatastoreEnterMaintenanceMode(self):
        '''Puts the datastore in maintenance mode. While this task is running and when the
        datastore is in maintenance mode, no virtual machines can be powered on and no
        provisioning operations can be performed on the datastore. Once the call
        completes, it is safe to remove datastore without disrupting any virtual
        machines.Puts the datastore in maintenance mode. While this task is running and
        when the datastore is in maintenance mode, no virtual machines can be powered
        on and no provisioning operations can be performed on the datastore. Once the
        call completes, it is safe to remove datastore without disrupting any virtual
        machines.
        
        '''
        return self.delegate("DatastoreEnterMaintenanceMode")()
    
    def DatastoreExitMaintenanceMode_Task(self):
        '''Takes the datastore out of maintenance mode.Takes the datastore out of
        maintenance mode.Takes the datastore out of maintenance mode.
        
        '''
        return self.delegate("DatastoreExitMaintenanceMode_Task")()
    
    def DestroyDatastore(self):
        '''<b>Deprecated.</b> <i>As of VI API 2.5 do not use this method. This method
        throws ResourceInUse. Datastores are automatically removed when no longer in
        use, so this method is unnecessary.</i> Removes a datastore. A datastore can be
        removed only if it is not currently used by any host or virtual machine.
        
        '''
        return self.delegate("DestroyDatastore")()
    
    def RefreshDatastore(self):
        '''Explicitly refreshes free-space and capacity values in summary and info.
        
        '''
        return self.delegate("RefreshDatastore")()
    
    def RefreshDatastoreStorageInfo(self):
        '''Refreshes all storage related information including free-space, capacity, and
        detailed usage of virtual machines. Updated values are available in summary and
        info.
        
        '''
        return self.delegate("RefreshDatastoreStorageInfo")()
    
    def RenameDatastore(self, newName):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use Rename_Task.</i> Renames a
        datastore.
        
        :param newName: The new name to assign to the datastore.
        
        '''
        return self.delegate("RenameDatastore")(newName)
    
    def UpdateVirtualMachineFiles_Task(self, mountPathDatastoreMapping):
        '''Update file paths embedded in virtual machine files on the datastore. This can
        be called after the file system corresponding to the datastore has been
        resignatured or remounted. Any MountPathDatastorePairs where the new path is
        the same as the original file path will be ignored.Update file paths embedded
        in virtual machine files on the datastore. This can be called after the file
        system corresponding to the datastore has been resignatured or remounted. Any
        MountPathDatastorePairs where the new path is the same as the original file
        path will be ignored.Update file paths embedded in virtual machine files on the
        datastore. This can be called after the file system corresponding to the
        datastore has been resignatured or remounted. Any MountPathDatastorePairs where
        the new path is the same as the original file path will be ignored.Update file
        paths embedded in virtual machine files on the datastore. This can be called
        after the file system corresponding to the datastore has been resignatured or
        remounted. Any MountPathDatastorePairs where the new path is the same as the
        original file path will be ignored.
        
        :param mountPathDatastoreMapping: Old mount path to datastore mapping.
        
        '''
        return self.delegate("UpdateVirtualMachineFiles_Task")(mountPathDatastoreMapping)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemProcess(Base):
    '''
    Commands relating to running processes.
    '''
    moid = 'ha-cli-handler-system-process'
    def list(self):
        '''
        List the VMkernel UserWorld processes currently on the host.
        :returns: vim.EsxCLI.system.process.list.UserWorld[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.process.List',
                            )   


from xml.sax.saxutils import escape, unescape
from lxml import etree

SOAP_VERSION = "urn:vim25/5.0"

def marshall_response(vim, response):
    from suds.sax.parser import Parser
    from suds.bindings.document import Document
    parser = Parser()
    document = parser.parse(string=response)
    obj = document.getChildren()[0]
    binding = Document(vim.client.wsdl)
    unmarshaller = binding.unmarshaller()
    marshalled_obj = unmarshaller.process(obj, None)
    return vim._parse_object_content(marshalled_obj)

def unmarshall_object(vim, obj):
    from suds.mx import Content
    from suds.sax.parser import Parser
    from suds.bindings.document import Document
    binding = Document(vim.client.wsdl)
    marshaller = binding.marshaller()
    content = Content(obj.__class__.__name__, obj)
    marshalled_obj = marshaller.process(content)
    return str(marshalled_obj)

def get_xml_from_value(vim, key, value):
    from lxml import etree
    import types
    BASIC_TYPES = (types.BooleanType, types.FloatType, types.IntType,
                   types.LongType, types.StringType, types.UnicodeType)
    if isinstance(value, BASIC_TYPES):
        element = etree.Element(key)
        element.text = str(value)
        return etree.tostring(element)
    if isinstance(value, etree._Element):
        return etree.tostring(value)
    return unmarshall_object(vim, value)

def generate_arguments(vim, **kwargs):
    from pyvisdk.do.reflect_managed_method_executer_soap_argument import ReflectManagedMethodExecuterSoapArgument
    sorted_keys = kwargs.keys()
    sorted_keys.sort()
    sorted_kwargs = [(key, kwargs[key]) for key in sorted_keys]
    return [ReflectManagedMethodExecuterSoapArgument(vim, key, escape(get_xml_from_value(vim, key, value)))
            for key, value in sorted_kwargs if value is not None]

def _extract_response(response):
    """:param response: `ReflectManagedMethodExecuterSoapResult`"""
    return unescape(response.response).replace('<NONE>', 'None')

def _extract_fault(response):
    """:param response: `ReflectManagedMethodExecuterSoapResult`"""
    return unescape(response.fault.faultDetail)

def execute_soap(vim, host, moid, method, **kwargs):
    from suds import WebFault
    response = host.RetrieveManagedMethodExecuter().executeSoap(moid=moid, version=SOAP_VERSION, method=method,
                                                                argument=generate_arguments(vim, **kwargs))
    if response is None:
        return

    if response.response is None and response.fault.faultMsg is not None:
        # HIPVM-457
        # response is ReflectManagedMethodExecuterSoapResult
        # fault is ReflectManagedMethodExecuterSoapFault
        raise WebFault(response.fault, None)
    return marshall_response(vim, response.response)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCorePlugin(Base):
    '''
    Operations pertaining to the pluggable storage architectures' plugins. The operation currently allowed is to list the available plugins on the system with the type of those plugins
    '''
    moid = 'ha-cli-handler-storage-core-plugin'
    def list(self, pluginclass=None):
        '''
        List loaded PSA plugins on the system.
        :param pluginclass: string, Indicate the class of plugin to limit the list to.  Allowed values are :
    Filter: Filter plugins
    MP: MultiPathing plugins
    VAAI: VAAI plugins
    all: All PSA Plugins (default)

        :returns: vim.EsxCLI.storage.core.plugin.list.PsaPlugin[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.plugin.List',
                            pluginclass=pluginclass,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseFeatureInfoUnit = Enum(
    'cpuCore',
    'cpuPackage',
    'host',
    'server',
    'vm',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationVirtualMachineName(vim, *args, **kwargs):
    '''Specifies that VirtualCenter should generate a virtual machine name from a base
    prefix comprising the virtual machine entity name. A number is appended, if
    necessary, to make it unique.Virtual machine names are unique across the set of
    hosts and virtual machines known to the VirtualCenter instance. VMware Tools
    reports the names of existing virtual machines.'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationVirtualMachineName')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class SmsProvider(BaseEntity):
    '''Provider interface for Storage Monitoring Service (SMS).'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.SmsProvider):
        super(SmsProvider, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def QueryProviderInfo(self):
        '''Get provider information.
        
        '''
        return self.delegate("QueryProviderInfo")()



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterVmHostRuleInfo(vim, *args, **kwargs):
    '''A ClusterVmHostRuleInfo object identifies virtual machines and host groups that
    determine virtual machine placement. The virtual machines and hosts referenced
    by a VM-Host rule must be in the same cluster.A VM-Host rule identifies the
    following groups.ClusterVmHostRuleInfo stores only the names of the relevant
    virtual machine and host groups. The group contents are stored in the virtual
    machine and host group objects.When you modify a VM-Host rule, only the fields
    that are specified are set.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterVmHostRuleInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'affineHostGroupName', 'antiAffineHostGroupName', 'vmGroupName', 'enabled',
        'inCompliance', 'key', 'mandatory', 'name', 'status', 'userCreated',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class SessionManager(BaseEntity):
    '''This managed object type includes methods for logging on and logging off
    clients, determining which clients are currently logged on, and forcing clients
    to log off.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.SessionManager):
        super(SessionManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def currentSession(self):
        '''This property contains information about the client's current session. If the
        client is not logged on, the value is null.'''
        return self.update('currentSession')
    @property
    def defaultLocale(self):
        '''This is the default server locale.'''
        return self.update('defaultLocale')
    @property
    def message(self):
        '''The system global message from the server.'''
        return self.update('message')
    @property
    def messageLocaleList(self):
        '''Provides the list of locales for which the server has localized messages.'''
        return self.update('messageLocaleList')
    @property
    def sessionList(self):
        '''The list of currently active sessions.'''
        return self.update('sessionList')
    @property
    def supportedLocaleList(self):
        '''Provides the list of locales that the server supports. Listing a locale ensures
        that some standardized information such as dates appear in the appropriate
        format. Other localized information, such as error messages, are displayed, if
        available. If localized information is not available, the message is returned
        using the system locale.'''
        return self.update('supportedLocaleList')

    
    
    def AcquireCloneTicket(self):
        '''Acquire a session-specific ticket string which can be used to clone the current
        session. The caller of this operation can pass the ticket value to another
        entity on the client. The recipient can then call CloneSession with the ticket
        string on an unauthenticated session and avoid having to re-enter
        credentials.Acquire a session-specific ticket string which can be used to clone
        the current session. The caller of this operation can pass the ticket value to
        another entity on the client. The recipient can then call CloneSession with the
        ticket string on an unauthenticated session and avoid having to re-enter
        credentials.Acquire a session-specific ticket string which can be used to clone
        the current session. The caller of this operation can pass the ticket value to
        another entity on the client. The recipient can then call CloneSession with the
        ticket string on an unauthenticated session and avoid having to re-enter
        credentials.
        
        '''
        return self.delegate("AcquireCloneTicket")()
    
    def AcquireGenericServiceTicket(self, spec):
        '''Creates and returns a one-time credential that may be used to make the
        specified request.
        
        :param spec: specification for the service request which will be invoked with the ticket.
        
        '''
        return self.delegate("AcquireGenericServiceTicket")(spec)
    
    def AcquireLocalTicket(self, userName):
        '''Acquires a one-time ticket for mutual authentication between a server and
        client.Acquires a one-time ticket for mutual authentication between a server
        and client.Acquires a one-time ticket for mutual authentication between a
        server and client.Acquires a one-time ticket for mutual authentication between
        a server and client.
        
        :param userName: User requesting one-time password.
        
        '''
        return self.delegate("AcquireLocalTicket")(userName)
    
    def CloneSession(self, cloneTicket):
        '''Clone the session specified by the clone ticket and associate it with the
        current connection. The current session will take on the identity and
        authorization level of the UserSession associated with the specified cloning
        ticket.See AcquireCloneTicket
        
        :param cloneTicket: ticket string acquired via AcquireCloneTicket.See AcquireCloneTicket
        
        '''
        return self.delegate("CloneSession")(cloneTicket)
    
    def ImpersonateUser(self, userName, locale=None):
        '''Converts current session to impersonate the specified user. The current session
        will take on the identity and authorization level of the user. That user must
        have a currently-active session. If the given userName is an extension key and
        this key does not overlap with a user name of any currently-active session, it
        will take on the identity and authorization level of that extension provided
        the current session has the same authorization level of that extension.
        
        :param userName: The user or extension key to impersonate.
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("ImpersonateUser")(userName, locale)
    
    def Login(self, userName, password, locale=None):
        '''Log on to the server. This method fails if the user name and password are
        incorrect, or if the user is valid but has no permissions granted.
        
        :param userName: The ID of the user who is logging on to the server.
        
        :param password: The password of the user who is logging on to the server.
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("Login")(userName, password, locale)
    
    def LoginBySSPI(self, base64Token, locale=None):
        '''Log on to the server using SSPI pass-through authentication.Log on to the
        server using SSPI pass-through authentication.Log on to the server using SSPI
        pass-through authentication.Log on to the server using SSPI pass-through
        authentication.Log on to the server using SSPI pass-through authentication.Log
        on to the server using SSPI pass-through authentication.
        
        :param base64Token: The partially formed context returned from InitializeSecurityContext().
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("LoginBySSPI")(base64Token, locale)
    
    def LoginExtensionByCertificate(self, extensionKey, locale=None):
        '''Creates a special privileged session that includes the Sessions.ImpersonateUser
        privilege. Requires that the client connect over SSL and provide an X.509
        certificate for which they hold the private key. The certificate must match the
        certificate used in an earlier call to SetExtensionCertificate.Creates a
        special privileged session that includes the Sessions.ImpersonateUser
        privilege. Requires that the client connect over SSL and provide an X.509
        certificate for which they hold the private key. The certificate must match the
        certificate used in an earlier call to SetExtensionCertificate.
        
        :param extensionKey: Key of extension that is logging in.
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("LoginExtensionByCertificate")(extensionKey, locale)
    
    def LoginExtensionBySubjectName(self, extensionKey, locale=None):
        '''Creates a special privileged session that includes the Sessions.ImpersonateUser
        privilege. Requires that the extension connected using SSL, with a certificate
        that has a subject name that matches the subject name registered for the
        extension.Creates a special privileged session that includes the
        Sessions.ImpersonateUser privilege. Requires that the extension connected using
        SSL, with a certificate that has a subject name that matches the subject name
        registered for the extension.
        
        :param extensionKey: Key of extension that is logging in.
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("LoginExtensionBySubjectName")(extensionKey, locale)
    
    def Logout(self):
        '''Log out and terminate the current session.
        
        '''
        return self.delegate("Logout")()
    
    def SessionIsActive(self, sessionID, userName):
        '''Validates that a currently-active session exists with the specified sessionID
        and userName associated with it. Returns true if session exists.
        
        :param sessionID: Session ID to validate.
        
        :param userName: User name to validate.
        
        '''
        return self.delegate("SessionIsActive")(sessionID, userName)
    
    def SetLocale(self, locale):
        '''Sets the session locale.
        
        :param locale: A two-character ISO-639 language ID (like "en") optionally followed by an underscore and a two-character ISO 3166 country ID (like "US").
        
        '''
        return self.delegate("SetLocale")(locale)
    
    def TerminateSession(self, sessionId):
        '''Log off and terminate the provided list of sessions.Log off and terminate the
        provided list of sessions.
        
        :param sessionId: A list of sessions to terminate.
        
        '''
        return self.delegate("TerminateSession")(sessionId)
    
    def UpdateServiceMessage(self, message):
        '''Updates the system global message. If not blank, the message is immediately
        displayed to currently logged-on users. When set, the message is shown by new
        clients upon logging in.
        
        :param message: The message to send. Newline characters may be included.
        
        '''
        return self.delegate("UpdateServiceMessage")(message)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostAutoStartManager(BaseEntity):
    '''The AutoStartManager allows clients to invoke and set up the auto-start/auto-
    stop order of virtual machines on a single host. Virtual machines configured to
    use auto-start are automatically started or stopped when the host is started or
    shut down. The AutoStartManager is available when clients connect directly to a
    host, such as an ESX Server machine or through VirtualCenter.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostAutoStartManager):
        super(HostAutoStartManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def config(self):
        ''''''
        return self.update('config')

    
    
    def AutoStartPowerOff(self):
        '''Powers-off virtual machines according to the current AutoStart configuration
        .Powers-off virtual machines according to the current AutoStart configuration.
        
        '''
        return self.delegate("AutoStartPowerOff")()
    
    def AutoStartPowerOn(self):
        '''Powers-on virtual machines according to the current AutoStart configuration
        .Powers-on virtual machines according to the current AutoStart configuration.
        
        '''
        return self.delegate("AutoStartPowerOn")()
    
    def ReconfigureAutostart(self, spec):
        '''Changes the power-on or power-off sequence and system defaults. The
        specification is an incremental change to the current configuration.Changes the
        power-on or power-off sequence and system defaults. The specification is an
        incremental change to the current configuration.Changes the power-on or power-
        off sequence and system defaults. The specification is an incremental change to
        the current configuration.Changes the power-on or power-off sequence and system
        defaults. The specification is an incremental change to the current
        configuration.
        
        :param spec: List of changes to defaults and auto-start/auto-stop order.
        
        '''
        return self.delegate("ReconfigureAutostart")(spec)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PropertySpec(vim, *args, **kwargs):
    '''Within a PropertyFilterSpec, A PropertySpec specifies which properties should
    be reported to the client for objects of the given managed object type that are
    visited and not skipped. One more subtle side effect is that if a managed
    object is visited and not skipped, but there is no PropertySpec associated with
    the managed object's type, the managed object will not be reported to the
    client.Also, the set of properties applicable to a given managed object type is
    the union of the properties implied by the PropertySpec objects even, in the
    case of a RetrieveResult, where there may be an applicable PropertySpec in more
    than one filter.'''

    obj = vim.client.factory.create('{urn:vim25}PropertySpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'type' ]
    optional = [ 'all', 'pathSet', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

GuestOsDescriptorSupportLevel = Enum(
    'experimental',
    'legacy',
    'supported',
    'terminated',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.network import Network

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class DistributedVirtualPortgroup(Network):
    '''The interface to the distributed virtual portgroup objects. This type
    represents both a group of ports that share the common network setting and a
    Network entity in the datacenter.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.DistributedVirtualPortgroup):
        super(DistributedVirtualPortgroup, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def config(self):
        '''The configuration of the portgroup.'''
        return self.update('config')
    @property
    def key(self):
        '''The generated UUID of the portgroup.'''
        return self.update('key')
    @property
    def portKeys(self):
        '''The port keys of those ports in the portgroup.'''
        return self.update('portKeys')

    
    
    def ReconfigureDVPortgroup_Task(self, spec):
        '''The DistributedVirtualPortgroups to be reconfiguredThe
        DistributedVirtualPortgroups to be reconfigured
        
        :param spec: 
        
        '''
        return self.delegate("ReconfigureDVPortgroup_Task")(spec)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VmFailedStartingSecondaryEventFailureReason = Enum(
    'incompatibleHost',
    'loginFailed',
    'migrateFailed',
    'registerVmFailed',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class StorageResourceManager(BaseEntity):
    '''This managed object type provides a way to configure resource usage for storage
    resources.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.StorageResourceManager):
        super(StorageResourceManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ApplyStorageDrsRecommendation_Task(self, key):
        '''Applies a recommendation from the recommendation list. Each recommendation can
        be applied only once. In the case of CreateVm and CloneVm a VirtualMachine is
        returned. Other workflows don't have a return value.Applies a recommendation
        from the recommendation list. Each recommendation can be applied only once. In
        the case of CreateVm and CloneVm a VirtualMachine is returned. Other workflows
        don't have a return value.Applies a recommendation from the recommendation
        list. Each recommendation can be applied only once. In the case of CreateVm and
        CloneVm a VirtualMachine is returned. Other workflows don't have a return
        value.
        
        :param key: The key fields of the Recommendations that are applied.
        
        '''
        return self.delegate("ApplyStorageDrsRecommendation_Task")(key)
    
    def ApplyStorageDrsRecommendationToPod_Task(self, pod, key):
        '''Applies a recommendation from the recommendation list. Each recommendation can
        be applied only once.Applies a recommendation from the recommendation list.
        Each recommendation can be applied only once.
        
        :param pod: The storage pod.
        
        :param key: The key field of the Recommendation.
        
        '''
        return self.delegate("ApplyStorageDrsRecommendationToPod_Task")(pod, key)
    
    def CancelStorageDrsRecommendation(self, key):
        '''Cancels a recommendation. Currently only initial placement recommendations can
        be cancelled. Migration recommendations cannot.
        
        :param key: The key field of the Recommendation.
        
        '''
        return self.delegate("CancelStorageDrsRecommendation")(key)
    
    def ConfigureDatastoreIORM_Task(self, datastore, spec):
        '''Changes configuration of storage I/O resource management for a given datastore.
        The changes are applied to all the backing storage devices for the datastore.
        Currently we only support storage I/O resource management on VMFS volumes. In
        order to enable storage I/O resource management on a datstore, we require that
        all the hosts that are attached to the datastore support this feature.Changes
        configuration of storage I/O resource management for a given datastore. The
        changes are applied to all the backing storage devices for the datastore.
        Currently we only support storage I/O resource management on VMFS volumes. In
        order to enable storage I/O resource management on a datstore, we require that
        all the hosts that are attached to the datastore support this feature.
        
        :param datastore: The datastore to be configured.
        
        :param spec: The configuration spec.
        
        '''
        return self.delegate("ConfigureDatastoreIORM_Task")(datastore, spec)
    
    def ConfigureStorageDrsForPod_Task(self, pod, spec, modify):
        '''Change the storage DRS configuration for a pod StoragePod.
        
        :param pod: The storage pod.
        
        :param spec: A set of storage Drs configuration changes to apply to the storage pod. The specification can be a complete set of changes or a partial set of changes, applied incrementally.
        
        :param modify: Flag to specify whether the specification ("spec") should be applied incrementally. If "modify" is false and the operation succeeds, then the configuration of the storage pod matches the specification exactly; in this case any unset portions of the specification will result in unset or default portions of the configuration.
        
        '''
        return self.delegate("ConfigureStorageDrsForPod_Task")(pod, spec, modify)
    
    def QueryIORMConfigOption(self, host):
        '''Query configuration options for storage I/O resource management.
        
        :param host: [in] - The host VC will forward the query to. This parameter is ignored by host if this method is called on a host directly.
        
        '''
        return self.delegate("QueryIORMConfigOption")(host)
    
    def RecommendDatastores(self, storageSpec):
        '''This method returns a StoragePlacementResult object. This API is intended to
        replace the following existing APIs for SDRS-enabled pods: CreateVm:
        StoragePlacementSpec::type == create = CreateVM_Task AddDisk:
        StoragePlacementSpec::type == reconfigure = ReconfigVM_Task RelocateVm:
        StoragePlacementSpec::type == relocate = RelocateVM_Task CloneVm:
        StoragePlacementSpec::type == clone = CloneVM_Task The PodSelectionSpec
        parameter in StoragePlacementSpec is required for all workflows. It specifies
        which SDRS-enabled pod the user has selected for the VM and/or for each disk.
        For CreateVm, RelocateVm and CloneVm, PodSelectionSpec.storagePod is the user
        selected SDRS pod for the VM, i.e., its system files. For all workflows,
        PodSelectionSpec.disk.storagePod is the user selected SDRS pod for the given
        disk. Note that a DiskLocator must be specified for each disk that the user
        requests to create, migrate or clone into an SDRS pod, even if it's the same
        pod as the VM or the user has manually selected a datastore within the pod. If
        the user has manually selected a datastore, the datastore must be specified in
        the workflow specific fields as described below. For CreateVm and AddDisk, the
        manually selected datastore must be specified in ConfigSpec.files or
        ConfigSpec.deviceChange.device.backing.datastore, the fields should will be
        unset if the user wants SDRS to recommend the datastore. For RelocateVm, the
        manually selected datastore must be specified in RelocateSpec.datastore or
        RelocateSpec.disk.datastore; the fields should be unset iff the user wants SDRS
        recommendations. For CloneVm, the manually selected datastore must be specified
        in CloneSpec.location.datastore or CloneSpec.location.disk[].datastore; the
        fields should be unset iff the user wants SDRS recommendations. The remaining
        expected input parameters in StoragePlacementSpec will be the same as those for
        the existing API as determined by StoragePlacementSpec::type. If a parameter is
        optional in the existing API, it will also be optional in the new API.This
        method returns a StoragePlacementResult object. This API is intended to replace
        the following existing APIs for SDRS-enabled pods: CreateVm:
        StoragePlacementSpec::type == create = CreateVM_Task AddDisk:
        StoragePlacementSpec::type == reconfigure = ReconfigVM_Task RelocateVm:
        StoragePlacementSpec::type == relocate = RelocateVM_Task CloneVm:
        StoragePlacementSpec::type == clone = CloneVM_Task The PodSelectionSpec
        parameter in StoragePlacementSpec is required for all workflows. It specifies
        which SDRS-enabled pod the user has selected for the VM and/or for each disk.
        For CreateVm, RelocateVm and CloneVm, PodSelectionSpec.storagePod is the user
        selected SDRS pod for the VM, i.e., its system files. For all workflows,
        PodSelectionSpec.disk.storagePod is the user selected SDRS pod for the given
        disk. Note that a DiskLocator must be specified for each disk that the user
        requests to create, migrate or clone into an SDRS pod, even if it's the same
        pod as the VM or the user has manually selected a datastore within the pod. If
        the user has manually selected a datastore, the datastore must be specified in
        the workflow specific fields as described below. For CreateVm and AddDisk, the
        manually selected datastore must be specified in ConfigSpec.files or
        ConfigSpec.deviceChange.device.backing.datastore, the fields should will be
        unset if the user wants SDRS to recommend the datastore. For RelocateVm, the
        manually selected datastore must be specified in RelocateSpec.datastore or
        RelocateSpec.disk.datastore; the fields should be unset iff the user wants SDRS
        recommendations. For CloneVm, the manually selected datastore must be specified
        in CloneSpec.location.datastore or CloneSpec.location.disk[].datastore; the
        fields should be unset iff the user wants SDRS recommendations. The remaining
        expected input parameters in StoragePlacementSpec will be the same as those for
        the existing API as determined by StoragePlacementSpec::type. If a parameter is
        optional in the existing API, it will also be optional in the new API.This
        method returns a StoragePlacementResult object. This API is intended to replace
        the following existing APIs for SDRS-enabled pods: CreateVm:
        StoragePlacementSpec::type == create = CreateVM_Task AddDisk:
        StoragePlacementSpec::type == reconfigure = ReconfigVM_Task RelocateVm:
        StoragePlacementSpec::type == relocate = RelocateVM_Task CloneVm:
        StoragePlacementSpec::type == clone = CloneVM_Task The PodSelectionSpec
        parameter in StoragePlacementSpec is required for all workflows. It specifies
        which SDRS-enabled pod the user has selected for the VM and/or for each disk.
        For CreateVm, RelocateVm and CloneVm, PodSelectionSpec.storagePod is the user
        selected SDRS pod for the VM, i.e., its system files. For all workflows,
        PodSelectionSpec.disk.storagePod is the user selected SDRS pod for the given
        disk. Note that a DiskLocator must be specified for each disk that the user
        requests to create, migrate or clone into an SDRS pod, even if it's the same
        pod as the VM or the user has manually selected a datastore within the pod. If
        the user has manually selected a datastore, the datastore must be specified in
        the workflow specific fields as described below. For CreateVm and AddDisk, the
        manually selected datastore must be specified in ConfigSpec.files or
        ConfigSpec.deviceChange.device.backing.datastore, the fields should will be
        unset if the user wants SDRS to recommend the datastore. For RelocateVm, the
        manually selected datastore must be specified in RelocateSpec.datastore or
        RelocateSpec.disk.datastore; the fields should be unset iff the user wants SDRS
        recommendations. For CloneVm, the manually selected datastore must be specified
        in CloneSpec.location.datastore or CloneSpec.location.disk[].datastore; the
        fields should be unset iff the user wants SDRS recommendations. The remaining
        expected input parameters in StoragePlacementSpec will be the same as those for
        the existing API as determined by StoragePlacementSpec::type. If a parameter is
        optional in the existing API, it will also be optional in the new API.This
        method returns a StoragePlacementResult object. This API is intended to replace
        the following existing APIs for SDRS-enabled pods: CreateVm:
        StoragePlacementSpec::type == create = CreateVM_Task AddDisk:
        StoragePlacementSpec::type == reconfigure = ReconfigVM_Task RelocateVm:
        StoragePlacementSpec::type == relocate = RelocateVM_Task CloneVm:
        StoragePlacementSpec::type == clone = CloneVM_Task The PodSelectionSpec
        parameter in StoragePlacementSpec is required for all workflows. It specifies
        which SDRS-enabled pod the user has selected for the VM and/or for each disk.
        For CreateVm, RelocateVm and CloneVm, PodSelectionSpec.storagePod is the user
        selected SDRS pod for the VM, i.e., its system files. For all workflows,
        PodSelectionSpec.disk.storagePod is the user selected SDRS pod for the given
        disk. Note that a DiskLocator must be specified for each disk that the user
        requests to create, migrate or clone into an SDRS pod, even if it's the same
        pod as the VM or the user has manually selected a datastore within the pod. If
        the user has manually selected a datastore, the datastore must be specified in
        the workflow specific fields as described below. For CreateVm and AddDisk, the
        manually selected datastore must be specified in ConfigSpec.files or
        ConfigSpec.deviceChange.device.backing.datastore, the fields should will be
        unset if the user wants SDRS to recommend the datastore. For RelocateVm, the
        manually selected datastore must be specified in RelocateSpec.datastore or
        RelocateSpec.disk.datastore; the fields should be unset iff the user wants SDRS
        recommendations. For CloneVm, the manually selected datastore must be specified
        in CloneSpec.location.datastore or CloneSpec.location.disk[].datastore; the
        fields should be unset iff the user wants SDRS recommendations. The remaining
        expected input parameters in StoragePlacementSpec will be the same as those for
        the existing API as determined by StoragePlacementSpec::type. If a parameter is
        optional in the existing API, it will also be optional in the new API.
        
        :param storageSpec: 
        
        '''
        return self.delegate("RecommendDatastores")(storageSpec)
    
    def RefreshStorageDrsRecommendation(self, pod):
        '''Make Storage DRS invoke again on the specified pod StoragePod and return a new
        list of recommendations. Concurrent "refresh" requests may be combined together
        and trigger only one Storage DRS invocation.
        
        :param pod: The storage pod. The recommendations generated is stored at PodStorageDrsEntry#recommendation.
        
        '''
        return self.delegate("RefreshStorageDrsRecommendation")(pod)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

OvfConsumerOstNodeType = Enum(
    'envelope',
    'virtualSystem',
    'virtualSystemCollection',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostPlugStoreTopologyPath(vim, *args, **kwargs):
    '''This data object type is an association class that describes a Path and its
    associated Device. A Path may be claimed by at most one Device.'''

    obj = vim.client.factory.create('{urn:vim25}HostPlugStoreTopologyPath')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'key', 'name' ]
    optional = [ 'adapter', 'channelNumber', 'device', 'lunNumber', 'target', 'targetNumber',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class GuestAuthManager(BaseEntity):
    '''AuthManager is the managed object that provides APIs to manipulate the guest
    operating authentication.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.GuestAuthManager):
        super(GuestAuthManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def AcquireCredentialsInGuest(self, vm, requestedAuth, sessionID=None):
        '''Authenticates in the guest and returns a GuestAuthentication object with the
        acquired credentials for use in subsequent guest operation calls.Authenticates
        in the guest and returns a GuestAuthentication object with the acquired
        credentials for use in subsequent guest operation calls.
        
        :param vm: MoRef of the VM to perform the operation on.
        
        :param requestedAuth: The guest authentication data used to acquire credentials. See GuestAuthentication.
        
        :param sessionID: The sessionID number should be provided only when responding to a server challenge. The sessionID number to be used with the challenge is found in the GuestAuthenticationChallenge object.
        
        '''
        return self.delegate("AcquireCredentialsInGuest")(vm, requestedAuth, sessionID)
    
    def ReleaseCredentialsInGuest(self, vm, auth):
        '''Releases session data and resources associated with a GuestAuthentication
        object returned by AcquireCredentialsInGuest.Releases session data and
        resources associated with a GuestAuthentication object returned by
        AcquireCredentialsInGuest.
        
        :param vm: MoRef of the VM to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        '''
        return self.delegate("ReleaseCredentialsInGuest")(vm, auth)
    
    def ValidateCredentialsInGuest(self, vm, auth):
        '''Validates the GuestAuthentication credentials.Validates the GuestAuthentication
        credentials.
        
        :param vm: MoRef of the VM to perform the operation on.
        
        :param auth: The guest authentication data. See GuestAuthentication.
        
        '''
        return self.delegate("ValidateCredentialsInGuest")(vm, auth)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class EnvironmentBrowser(BaseEntity):
    '''This managed object type provides access to the environment that a
    ComputeResource presents for creating and configuring a virtual machine.The
    environment consists of three main components:* The virtual machine
    configuration options. Each vim.vm.ConfigOption describes the execution
    environment for a virtual machine, the particular set of virtual hardware that
    is supported. A ComputeResource might support multiple sets. Access is provided
    through the configOptionDescriptor property and the QueryConfigOption
    operation. * The supported device targets. Each virtual device specified in the
    virtual machine needs to be hooked up to a "physical" counterpart. For
    networks, this means choosing a network name; for a virtual CD-rom this might
    be an ISO image, etc. The environment browser provides access to the device
    targets through the QueryConfigTarget operation. * Storage locations and files.
    A selection of locations where the virtual machine files can be stored, and the
    possibility to browse for existing virtual disks and ISO images. The datastore
    browser, provided by the datastoreBrowser property, provides access to the
    contents of one or more datastores. The items in a datastore are files that
    contain configuration, virtual disk, and the other data associated with a
    virtual machine. * The capabilities supported by the ComputeResource to which
    the virtual machine belongs.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.EnvironmentBrowser):
        super(EnvironmentBrowser, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def datastoreBrowser(self):
        '''DatastoreBrowser to browse datastores that are available on this entity.'''
        return self.update('datastoreBrowser')

    
    
    def QueryConfigOption(self, key=None, host=None):
        '''Query for a specific virtual machine configuration option (the
        ConfigOption).Query for a specific virtual machine configuration option (the
        ConfigOption).Query for a specific virtual machine configuration option (the
        ConfigOption).
        
        :param key: The key found in the VirtualMachineConfigOptionDescriptor, obtained by invoking the QueryConfigOptionDescriptor operation.
        
        :param host: The host whose ConfigOption is requested.
        
        '''
        return self.delegate("QueryConfigOption")(key, host)
    
    def QueryConfigOptionDescriptor(self):
        '''The list of ConfigOption keys available on this entity.
        
        '''
        return self.delegate("QueryConfigOptionDescriptor")()
    
    def QueryConfigTarget(self, host=None):
        '''Queries for information about a specific target, a "physical" device that can
        be used to back virtual devices. The ConfigTarget that is returned specifies
        the set of values that can be used in the device backings to connect the
        virtual machine to physical (or logical) host devices.Queries for information
        about a specific target, a "physical" device that can be used to back virtual
        devices. The ConfigTarget that is returned specifies the set of values that can
        be used in the device backings to connect the virtual machine to physical (or
        logical) host devices.Queries for information about a specific target, a
        "physical" device that can be used to back virtual devices. The ConfigTarget
        that is returned specifies the set of values that can be used in the device
        backings to connect the virtual machine to physical (or logical) host devices.
        
        :param host: If specified, the host whose default BackingInfo is requested.
        
        '''
        return self.delegate("QueryConfigTarget")(host)
    
    def QueryTargetCapabilities(self, host=None):
        '''Queries for information on the capabilities supported by the ComputeResource
        associated with the EnvironmentBrowser.Queries for information on the
        capabilities supported by the ComputeResource associated with the
        EnvironmentBrowser.Queries for information on the capabilities supported by the
        ComputeResource associated with the EnvironmentBrowser.
        
        :param host: If specified, the host whose capabilities are requested.
        
        '''
        return self.delegate("QueryTargetCapabilities")(host)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VasaProviderStatus = Enum(
    'offline',
    'online',
    'syncError',
    'unknown',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VMotionCompatibilityType = Enum(
    'cpu',
    'software',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDeviceConnectInfoStatus = Enum(
    'ok',
    'recoverableError',
    'unrecoverableError',
    'untried',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostMountMode = Enum(
    'readOnly',
    'readWrite',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PhysicalNicVmDirectPathGen2SupportedMode = Enum(
    'upt',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFenceNetworkPort(Base):
    '''
    Commands to list fence network port information
    '''
    moid = 'ha-cli-handler-network-fence-network-port'
    def list(self, fenceid, vdsname):
        '''
        Get all fence port info on the fence network.
        :param fenceid: long, The fence id used to retrieve fence info.
        :param vdsname: string, The vds name used to retrieve fence info.
        :returns: vim.EsxCLI.network.fence.network.port.list.FencePortList
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.fence.network.port.List',
                            fenceid=fenceid,
                            vdsname=vdsname,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSettingsKernel(Base):
    '''
    Operations that allow viewing and manipulation of system kernel settings.
    '''
    moid = 'ha-cli-handler-system-settings-kernel'
    def set(self, setting, value):
        '''
        Set a VMKernel setting.
        :param setting: string, The name of the VMKernel setting to set.
        :param value: string, The value to set the setting to.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.kernel.Set',
                            setting=setting,
                            value=value,
                            )
    def list(self, option=None):
        '''
        List VMkernel kernel settings.
        :param option: string, The name of the VMkernel kernel setting to get.
        :returns: vim.EsxCLI.system.settings.kernel.list.SettingsKernel[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.kernel.List',
                            option=option,
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class HardwarePlatform(Base):
    '''
    Platform information.
    '''
    moid = 'ha-cli-handler-hardware-platform'
    def get(self):
        '''
        Get information about the platform
        :returns: vim.EsxCLI.hardware.platform.get.PlatformGet
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.hardware.platform.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationLicenseFilePrintData(vim, *args, **kwargs):
    '''The LicenseFilePrintData type maps directly to the LicenseFilePrintData key in
    the answer file. These values are transferred into the file that VirtualCenter
    stores on the target virtual disk. For more detailed information, see the
    document . LicenseFilePrintData provides licensing information for Windows
    server operating systems.'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationLicenseFilePrintData')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'autoMode' ]
    optional = [ 'autoUsers', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from .base import Base
from .generator import Generator

class EsxCLI(Base):
    def __init__(self, client, host):
        super(EsxCLI, self).__init__(client, host)

    def _get_init_args(self):
        return self._client, self._host

    def get(self, namespace):
        from .handlers import import_handler
        return import_handler(''.join([item.capitalize() for item in namespace.split('.')]))(*self._get_init_args())




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DeviceNotSupportedReason = Enum(
    'guest',
    'host',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineConfigInfo(vim, *args, **kwargs):
    '''The ConfigInfo data object type encapsulates the configuration settings and
    virtual hardware for a virtual machine. This type holds all the information
    that is present in the .vmx configuration file for the virtual machine.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineConfigInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 13:
        raise IndexError('Expected at least 14 arguments got: %d' % len(args))

    required = [ 'alternateGuestName', 'changeVersion', 'defaultPowerOps', 'files', 'flags',
        'guestFullName', 'guestId', 'hardware', 'modified', 'name', 'template', 'uuid',
        'version' ]
    optional = [ 'annotation', 'bootOptions', 'changeTrackingEnabled', 'consolePreferences',
        'cpuAffinity', 'cpuAllocation', 'cpuFeatureMask', 'cpuHotAddEnabled',
        'cpuHotRemoveEnabled', 'datastoreUrl', 'extraConfig', 'firmware', 'ftInfo',
        'guestAutoLockEnabled', 'hotPlugMemoryIncrementSize', 'hotPlugMemoryLimit',
        'initialOverhead', 'instanceUuid', 'locationId', 'managedBy',
        'maxMksConnections', 'memoryAffinity', 'memoryAllocation',
        'memoryHotAddEnabled', 'memoryReservationLockedToMax', 'networkShaper',
        'npivDesiredNodeWwns', 'npivDesiredPortWwns', 'npivNodeWorldWideName',
        'npivOnNonRdmDisks', 'npivPortWorldWideName', 'npivTemporaryDisabled',
        'npivWorldWideNameType', 'swapPlacement', 'tools', 'vAppConfig',
        'vAssertsEnabled', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostPatchManagerStatusPrerequisitePatch(vim, *args, **kwargs):
    '''Updates that are required to be installed before this update can be installed
    on the server. In addition to being installed on the server, an update can have
    additional requirement on the server or services running on the server
    pertaining to the prerequisite update.'''

    obj = vim.client.factory.create('{urn:vim25}HostPatchManagerStatusPrerequisitePatch')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'id' ]
    optional = [ 'installState', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

StorageDrsPodConfigInfoBehavior = Enum(
    'automated',
    'manual',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiSessionConnection(Base):
    '''
    Operations that can be performed on iSCSI connections
    '''
    moid = 'ha-cli-handler-iscsi-session-connection'
    def list(self, adapter=None, cid=None, isid=None, name=None):
        '''
        List iSCSI connections.
        :param adapter: string, The iSCSI adapter name.
        :param cid: string, The iSCSI connection identifier(CID).
        :param isid: string, The iSCSI session identifier(ISID).
        :param name: string, The iSCSI target name.
        :returns: vim.EsxCLI.iscsi.session.connection.list.Connection[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.session.connection.List',
                            adapter=adapter,
                            cid=cid,
                            isid=isid,
                            name=name,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DistributedVirtualPortgroupPortgroupType = Enum(
    'earlyBinding',
    'ephemeral',
    'lateBinding',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpInterface(Base):
    '''
    Operations having to do with the creation, management and deletion of VMkernel network interfaces (vmknic).
    '''
    moid = 'ha-cli-handler-network-ip-interface'
    def add(self, portgroupname, interfacename=None, macaddress=None, mtu=None):
        '''
        Add a new VMkernel network interface.
        :param interfacename: string, The name of the VMkernel network interface to create. This name must be in the form vmkX, where X is a number 0-99
        :param macaddress: string, Set the MAC address for the newly created VMkernel network interface.
        :param mtu: long, Set the MTU setting for a given VMkernel network interface on creation
        :param portgroupname: string, The name of the port group to add this VMkernel network interface to. This option is required.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.Add',
                            interfacename=interfacename,
                            macaddress=macaddress,
                            mtu=mtu,
                            portgroupname=portgroupname,
                            )
    def set(self, interfacename, enabled=None, mtu=None):
        '''
        This command sets the enabled status and MTU size of a given IP interface 
        :param enabled: boolean, Set to true to enable the interface, set to false to disable it.
        :param interfacename: string, The name of the interface to apply the configurations.
        :param mtu: long, The MTU size of the IP interface. 
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.Set',
                            enabled=enabled,
                            interfacename=interfacename,
                            mtu=mtu,
                            )
    def list(self):
        '''
        This command will list the VMkernel network interfaces currently known to the system.
        :returns: vim.EsxCLI.network.ip.interface.list.NetworkInterface[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.List',
                            )
    def remove(self, interfacename):
        '''
        Remove a new VMkernel network interface from the ESXi host.
        :param interfacename: string, The name of the VMkernel network interface to remove. This name must be in the form vmkX, where X is a number 0-99
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.Remove',
                            interfacename=interfacename,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostDigestInfoDigestMethodType = Enum(
    'MD5',
    'SHA1',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SnapshotMoveFromNonHomeNotSupported(vim, *args, **kwargs):
    '''An attempt is being made to move a virtual machine's disk that has associated
    snapshots, and preserving the snapshots is not supported by the host because
    the disk is currently located somewhere other than the virtual machine's home
    datastore.'''

    obj = vim.client.factory.create('{urn:vim25}SnapshotMoveFromNonHomeNotSupported')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj
